r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Api
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import datetime
from typing import Any, Dict, Optional, Union
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class StreamInstance(InstanceResource):

    class Status(object):
        IN_PROGRESS = "in-progress"
        STOPPED = "stopped"

    class Track(object):
        INBOUND_TRACK = "inbound_track"
        OUTBOUND_TRACK = "outbound_track"
        BOTH_TRACKS = "both_tracks"

    class UpdateStatus(object):
        STOPPED = "stopped"

    """
    :ivar sid: The SID of the Stream resource.
    :ivar account_sid: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created this Stream resource.
    :ivar call_sid: The SID of the [Call](https://www.twilio.com/docs/voice/api/call-resource) the Stream resource is associated with.
    :ivar name: The user-specified name of this Stream, if one was given when the Stream was created. This can be used to stop the Stream.
    :ivar status: 
    :ivar date_updated: The date and time in GMT that this resource was last updated, specified in [RFC 2822](https://www.ietf.org/rfc/rfc2822.txt) format.
    :ivar uri: The URI of the resource, relative to `https://api.twilio.com`.
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        account_sid: str,
        call_sid: str,
        sid: Optional[str] = None,
    ):
        super().__init__(version)

        self.sid: Optional[str] = payload.get("sid")
        self.account_sid: Optional[str] = payload.get("account_sid")
        self.call_sid: Optional[str] = payload.get("call_sid")
        self.name: Optional[str] = payload.get("name")
        self.status: Optional["StreamInstance.Status"] = payload.get("status")
        self.date_updated: Optional[datetime] = deserialize.rfc2822_datetime(
            payload.get("date_updated")
        )
        self.uri: Optional[str] = payload.get("uri")

        self._solution = {
            "account_sid": account_sid,
            "call_sid": call_sid,
            "sid": sid or self.sid,
        }
        self._context: Optional[StreamContext] = None

    @property
    def _proxy(self) -> "StreamContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: StreamContext for this StreamInstance
        """
        if self._context is None:
            self._context = StreamContext(
                self._version,
                account_sid=self._solution["account_sid"],
                call_sid=self._solution["call_sid"],
                sid=self._solution["sid"],
            )
        return self._context

    def update(self, status: "StreamInstance.UpdateStatus") -> "StreamInstance":
        """
        Update the StreamInstance

        :param status:

        :returns: The updated StreamInstance
        """
        return self._proxy.update(
            status=status,
        )

    async def update_async(
        self, status: "StreamInstance.UpdateStatus"
    ) -> "StreamInstance":
        """
        Asynchronous coroutine to update the StreamInstance

        :param status:

        :returns: The updated StreamInstance
        """
        return await self._proxy.update_async(
            status=status,
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Api.V2010.StreamInstance {}>".format(context)


class StreamContext(InstanceContext):

    def __init__(self, version: Version, account_sid: str, call_sid: str, sid: str):
        """
        Initialize the StreamContext

        :param version: Version that contains the resource
        :param account_sid: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created this Stream resource.
        :param call_sid: The SID of the [Call](https://www.twilio.com/docs/voice/api/call-resource) the Stream resource is associated with.
        :param sid: The SID or the `name` of the Stream resource to be stopped
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "account_sid": account_sid,
            "call_sid": call_sid,
            "sid": sid,
        }
        self._uri = (
            "/Accounts/{account_sid}/Calls/{call_sid}/Streams/{sid}.json".format(
                **self._solution
            )
        )

    def update(self, status: "StreamInstance.UpdateStatus") -> StreamInstance:
        """
        Update the StreamInstance

        :param status:

        :returns: The updated StreamInstance
        """

        data = values.of(
            {
                "Status": status,
            }
        )
        headers = values.of({})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = self._version.update(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            call_sid=self._solution["call_sid"],
            sid=self._solution["sid"],
        )

    async def update_async(
        self, status: "StreamInstance.UpdateStatus"
    ) -> StreamInstance:
        """
        Asynchronous coroutine to update the StreamInstance

        :param status:

        :returns: The updated StreamInstance
        """

        data = values.of(
            {
                "Status": status,
            }
        )
        headers = values.of({})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = await self._version.update_async(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            call_sid=self._solution["call_sid"],
            sid=self._solution["sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Api.V2010.StreamContext {}>".format(context)


class StreamList(ListResource):

    def __init__(self, version: Version, account_sid: str, call_sid: str):
        """
        Initialize the StreamList

        :param version: Version that contains the resource
        :param account_sid: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created this Stream resource.
        :param call_sid: The SID of the [Call](https://www.twilio.com/docs/voice/api/call-resource) the Stream resource is associated with.

        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "account_sid": account_sid,
            "call_sid": call_sid,
        }
        self._uri = "/Accounts/{account_sid}/Calls/{call_sid}/Streams.json".format(
            **self._solution
        )

    def create(
        self,
        url: str,
        name: Union[str, object] = values.unset,
        track: Union["StreamInstance.Track", object] = values.unset,
        status_callback: Union[str, object] = values.unset,
        status_callback_method: Union[str, object] = values.unset,
        parameter1_name: Union[str, object] = values.unset,
        parameter1_value: Union[str, object] = values.unset,
        parameter2_name: Union[str, object] = values.unset,
        parameter2_value: Union[str, object] = values.unset,
        parameter3_name: Union[str, object] = values.unset,
        parameter3_value: Union[str, object] = values.unset,
        parameter4_name: Union[str, object] = values.unset,
        parameter4_value: Union[str, object] = values.unset,
        parameter5_name: Union[str, object] = values.unset,
        parameter5_value: Union[str, object] = values.unset,
        parameter6_name: Union[str, object] = values.unset,
        parameter6_value: Union[str, object] = values.unset,
        parameter7_name: Union[str, object] = values.unset,
        parameter7_value: Union[str, object] = values.unset,
        parameter8_name: Union[str, object] = values.unset,
        parameter8_value: Union[str, object] = values.unset,
        parameter9_name: Union[str, object] = values.unset,
        parameter9_value: Union[str, object] = values.unset,
        parameter10_name: Union[str, object] = values.unset,
        parameter10_value: Union[str, object] = values.unset,
        parameter11_name: Union[str, object] = values.unset,
        parameter11_value: Union[str, object] = values.unset,
        parameter12_name: Union[str, object] = values.unset,
        parameter12_value: Union[str, object] = values.unset,
        parameter13_name: Union[str, object] = values.unset,
        parameter13_value: Union[str, object] = values.unset,
        parameter14_name: Union[str, object] = values.unset,
        parameter14_value: Union[str, object] = values.unset,
        parameter15_name: Union[str, object] = values.unset,
        parameter15_value: Union[str, object] = values.unset,
        parameter16_name: Union[str, object] = values.unset,
        parameter16_value: Union[str, object] = values.unset,
        parameter17_name: Union[str, object] = values.unset,
        parameter17_value: Union[str, object] = values.unset,
        parameter18_name: Union[str, object] = values.unset,
        parameter18_value: Union[str, object] = values.unset,
        parameter19_name: Union[str, object] = values.unset,
        parameter19_value: Union[str, object] = values.unset,
        parameter20_name: Union[str, object] = values.unset,
        parameter20_value: Union[str, object] = values.unset,
        parameter21_name: Union[str, object] = values.unset,
        parameter21_value: Union[str, object] = values.unset,
        parameter22_name: Union[str, object] = values.unset,
        parameter22_value: Union[str, object] = values.unset,
        parameter23_name: Union[str, object] = values.unset,
        parameter23_value: Union[str, object] = values.unset,
        parameter24_name: Union[str, object] = values.unset,
        parameter24_value: Union[str, object] = values.unset,
        parameter25_name: Union[str, object] = values.unset,
        parameter25_value: Union[str, object] = values.unset,
        parameter26_name: Union[str, object] = values.unset,
        parameter26_value: Union[str, object] = values.unset,
        parameter27_name: Union[str, object] = values.unset,
        parameter27_value: Union[str, object] = values.unset,
        parameter28_name: Union[str, object] = values.unset,
        parameter28_value: Union[str, object] = values.unset,
        parameter29_name: Union[str, object] = values.unset,
        parameter29_value: Union[str, object] = values.unset,
        parameter30_name: Union[str, object] = values.unset,
        parameter30_value: Union[str, object] = values.unset,
        parameter31_name: Union[str, object] = values.unset,
        parameter31_value: Union[str, object] = values.unset,
        parameter32_name: Union[str, object] = values.unset,
        parameter32_value: Union[str, object] = values.unset,
        parameter33_name: Union[str, object] = values.unset,
        parameter33_value: Union[str, object] = values.unset,
        parameter34_name: Union[str, object] = values.unset,
        parameter34_value: Union[str, object] = values.unset,
        parameter35_name: Union[str, object] = values.unset,
        parameter35_value: Union[str, object] = values.unset,
        parameter36_name: Union[str, object] = values.unset,
        parameter36_value: Union[str, object] = values.unset,
        parameter37_name: Union[str, object] = values.unset,
        parameter37_value: Union[str, object] = values.unset,
        parameter38_name: Union[str, object] = values.unset,
        parameter38_value: Union[str, object] = values.unset,
        parameter39_name: Union[str, object] = values.unset,
        parameter39_value: Union[str, object] = values.unset,
        parameter40_name: Union[str, object] = values.unset,
        parameter40_value: Union[str, object] = values.unset,
        parameter41_name: Union[str, object] = values.unset,
        parameter41_value: Union[str, object] = values.unset,
        parameter42_name: Union[str, object] = values.unset,
        parameter42_value: Union[str, object] = values.unset,
        parameter43_name: Union[str, object] = values.unset,
        parameter43_value: Union[str, object] = values.unset,
        parameter44_name: Union[str, object] = values.unset,
        parameter44_value: Union[str, object] = values.unset,
        parameter45_name: Union[str, object] = values.unset,
        parameter45_value: Union[str, object] = values.unset,
        parameter46_name: Union[str, object] = values.unset,
        parameter46_value: Union[str, object] = values.unset,
        parameter47_name: Union[str, object] = values.unset,
        parameter47_value: Union[str, object] = values.unset,
        parameter48_name: Union[str, object] = values.unset,
        parameter48_value: Union[str, object] = values.unset,
        parameter49_name: Union[str, object] = values.unset,
        parameter49_value: Union[str, object] = values.unset,
        parameter50_name: Union[str, object] = values.unset,
        parameter50_value: Union[str, object] = values.unset,
        parameter51_name: Union[str, object] = values.unset,
        parameter51_value: Union[str, object] = values.unset,
        parameter52_name: Union[str, object] = values.unset,
        parameter52_value: Union[str, object] = values.unset,
        parameter53_name: Union[str, object] = values.unset,
        parameter53_value: Union[str, object] = values.unset,
        parameter54_name: Union[str, object] = values.unset,
        parameter54_value: Union[str, object] = values.unset,
        parameter55_name: Union[str, object] = values.unset,
        parameter55_value: Union[str, object] = values.unset,
        parameter56_name: Union[str, object] = values.unset,
        parameter56_value: Union[str, object] = values.unset,
        parameter57_name: Union[str, object] = values.unset,
        parameter57_value: Union[str, object] = values.unset,
        parameter58_name: Union[str, object] = values.unset,
        parameter58_value: Union[str, object] = values.unset,
        parameter59_name: Union[str, object] = values.unset,
        parameter59_value: Union[str, object] = values.unset,
        parameter60_name: Union[str, object] = values.unset,
        parameter60_value: Union[str, object] = values.unset,
        parameter61_name: Union[str, object] = values.unset,
        parameter61_value: Union[str, object] = values.unset,
        parameter62_name: Union[str, object] = values.unset,
        parameter62_value: Union[str, object] = values.unset,
        parameter63_name: Union[str, object] = values.unset,
        parameter63_value: Union[str, object] = values.unset,
        parameter64_name: Union[str, object] = values.unset,
        parameter64_value: Union[str, object] = values.unset,
        parameter65_name: Union[str, object] = values.unset,
        parameter65_value: Union[str, object] = values.unset,
        parameter66_name: Union[str, object] = values.unset,
        parameter66_value: Union[str, object] = values.unset,
        parameter67_name: Union[str, object] = values.unset,
        parameter67_value: Union[str, object] = values.unset,
        parameter68_name: Union[str, object] = values.unset,
        parameter68_value: Union[str, object] = values.unset,
        parameter69_name: Union[str, object] = values.unset,
        parameter69_value: Union[str, object] = values.unset,
        parameter70_name: Union[str, object] = values.unset,
        parameter70_value: Union[str, object] = values.unset,
        parameter71_name: Union[str, object] = values.unset,
        parameter71_value: Union[str, object] = values.unset,
        parameter72_name: Union[str, object] = values.unset,
        parameter72_value: Union[str, object] = values.unset,
        parameter73_name: Union[str, object] = values.unset,
        parameter73_value: Union[str, object] = values.unset,
        parameter74_name: Union[str, object] = values.unset,
        parameter74_value: Union[str, object] = values.unset,
        parameter75_name: Union[str, object] = values.unset,
        parameter75_value: Union[str, object] = values.unset,
        parameter76_name: Union[str, object] = values.unset,
        parameter76_value: Union[str, object] = values.unset,
        parameter77_name: Union[str, object] = values.unset,
        parameter77_value: Union[str, object] = values.unset,
        parameter78_name: Union[str, object] = values.unset,
        parameter78_value: Union[str, object] = values.unset,
        parameter79_name: Union[str, object] = values.unset,
        parameter79_value: Union[str, object] = values.unset,
        parameter80_name: Union[str, object] = values.unset,
        parameter80_value: Union[str, object] = values.unset,
        parameter81_name: Union[str, object] = values.unset,
        parameter81_value: Union[str, object] = values.unset,
        parameter82_name: Union[str, object] = values.unset,
        parameter82_value: Union[str, object] = values.unset,
        parameter83_name: Union[str, object] = values.unset,
        parameter83_value: Union[str, object] = values.unset,
        parameter84_name: Union[str, object] = values.unset,
        parameter84_value: Union[str, object] = values.unset,
        parameter85_name: Union[str, object] = values.unset,
        parameter85_value: Union[str, object] = values.unset,
        parameter86_name: Union[str, object] = values.unset,
        parameter86_value: Union[str, object] = values.unset,
        parameter87_name: Union[str, object] = values.unset,
        parameter87_value: Union[str, object] = values.unset,
        parameter88_name: Union[str, object] = values.unset,
        parameter88_value: Union[str, object] = values.unset,
        parameter89_name: Union[str, object] = values.unset,
        parameter89_value: Union[str, object] = values.unset,
        parameter90_name: Union[str, object] = values.unset,
        parameter90_value: Union[str, object] = values.unset,
        parameter91_name: Union[str, object] = values.unset,
        parameter91_value: Union[str, object] = values.unset,
        parameter92_name: Union[str, object] = values.unset,
        parameter92_value: Union[str, object] = values.unset,
        parameter93_name: Union[str, object] = values.unset,
        parameter93_value: Union[str, object] = values.unset,
        parameter94_name: Union[str, object] = values.unset,
        parameter94_value: Union[str, object] = values.unset,
        parameter95_name: Union[str, object] = values.unset,
        parameter95_value: Union[str, object] = values.unset,
        parameter96_name: Union[str, object] = values.unset,
        parameter96_value: Union[str, object] = values.unset,
        parameter97_name: Union[str, object] = values.unset,
        parameter97_value: Union[str, object] = values.unset,
        parameter98_name: Union[str, object] = values.unset,
        parameter98_value: Union[str, object] = values.unset,
        parameter99_name: Union[str, object] = values.unset,
        parameter99_value: Union[str, object] = values.unset,
    ) -> StreamInstance:
        """
        Create the StreamInstance

        :param url: Relative or absolute URL where WebSocket connection will be established.
        :param name: The user-specified name of this Stream, if one was given when the Stream was created. This can be used to stop the Stream.
        :param track:
        :param status_callback: Absolute URL to which Twilio sends status callback HTTP requests.
        :param status_callback_method: The HTTP method Twilio uses when sending `status_callback` requests. Possible values are `GET` and `POST`. Default is `POST`.
        :param parameter1_name: Parameter name
        :param parameter1_value: Parameter value
        :param parameter2_name: Parameter name
        :param parameter2_value: Parameter value
        :param parameter3_name: Parameter name
        :param parameter3_value: Parameter value
        :param parameter4_name: Parameter name
        :param parameter4_value: Parameter value
        :param parameter5_name: Parameter name
        :param parameter5_value: Parameter value
        :param parameter6_name: Parameter name
        :param parameter6_value: Parameter value
        :param parameter7_name: Parameter name
        :param parameter7_value: Parameter value
        :param parameter8_name: Parameter name
        :param parameter8_value: Parameter value
        :param parameter9_name: Parameter name
        :param parameter9_value: Parameter value
        :param parameter10_name: Parameter name
        :param parameter10_value: Parameter value
        :param parameter11_name: Parameter name
        :param parameter11_value: Parameter value
        :param parameter12_name: Parameter name
        :param parameter12_value: Parameter value
        :param parameter13_name: Parameter name
        :param parameter13_value: Parameter value
        :param parameter14_name: Parameter name
        :param parameter14_value: Parameter value
        :param parameter15_name: Parameter name
        :param parameter15_value: Parameter value
        :param parameter16_name: Parameter name
        :param parameter16_value: Parameter value
        :param parameter17_name: Parameter name
        :param parameter17_value: Parameter value
        :param parameter18_name: Parameter name
        :param parameter18_value: Parameter value
        :param parameter19_name: Parameter name
        :param parameter19_value: Parameter value
        :param parameter20_name: Parameter name
        :param parameter20_value: Parameter value
        :param parameter21_name: Parameter name
        :param parameter21_value: Parameter value
        :param parameter22_name: Parameter name
        :param parameter22_value: Parameter value
        :param parameter23_name: Parameter name
        :param parameter23_value: Parameter value
        :param parameter24_name: Parameter name
        :param parameter24_value: Parameter value
        :param parameter25_name: Parameter name
        :param parameter25_value: Parameter value
        :param parameter26_name: Parameter name
        :param parameter26_value: Parameter value
        :param parameter27_name: Parameter name
        :param parameter27_value: Parameter value
        :param parameter28_name: Parameter name
        :param parameter28_value: Parameter value
        :param parameter29_name: Parameter name
        :param parameter29_value: Parameter value
        :param parameter30_name: Parameter name
        :param parameter30_value: Parameter value
        :param parameter31_name: Parameter name
        :param parameter31_value: Parameter value
        :param parameter32_name: Parameter name
        :param parameter32_value: Parameter value
        :param parameter33_name: Parameter name
        :param parameter33_value: Parameter value
        :param parameter34_name: Parameter name
        :param parameter34_value: Parameter value
        :param parameter35_name: Parameter name
        :param parameter35_value: Parameter value
        :param parameter36_name: Parameter name
        :param parameter36_value: Parameter value
        :param parameter37_name: Parameter name
        :param parameter37_value: Parameter value
        :param parameter38_name: Parameter name
        :param parameter38_value: Parameter value
        :param parameter39_name: Parameter name
        :param parameter39_value: Parameter value
        :param parameter40_name: Parameter name
        :param parameter40_value: Parameter value
        :param parameter41_name: Parameter name
        :param parameter41_value: Parameter value
        :param parameter42_name: Parameter name
        :param parameter42_value: Parameter value
        :param parameter43_name: Parameter name
        :param parameter43_value: Parameter value
        :param parameter44_name: Parameter name
        :param parameter44_value: Parameter value
        :param parameter45_name: Parameter name
        :param parameter45_value: Parameter value
        :param parameter46_name: Parameter name
        :param parameter46_value: Parameter value
        :param parameter47_name: Parameter name
        :param parameter47_value: Parameter value
        :param parameter48_name: Parameter name
        :param parameter48_value: Parameter value
        :param parameter49_name: Parameter name
        :param parameter49_value: Parameter value
        :param parameter50_name: Parameter name
        :param parameter50_value: Parameter value
        :param parameter51_name: Parameter name
        :param parameter51_value: Parameter value
        :param parameter52_name: Parameter name
        :param parameter52_value: Parameter value
        :param parameter53_name: Parameter name
        :param parameter53_value: Parameter value
        :param parameter54_name: Parameter name
        :param parameter54_value: Parameter value
        :param parameter55_name: Parameter name
        :param parameter55_value: Parameter value
        :param parameter56_name: Parameter name
        :param parameter56_value: Parameter value
        :param parameter57_name: Parameter name
        :param parameter57_value: Parameter value
        :param parameter58_name: Parameter name
        :param parameter58_value: Parameter value
        :param parameter59_name: Parameter name
        :param parameter59_value: Parameter value
        :param parameter60_name: Parameter name
        :param parameter60_value: Parameter value
        :param parameter61_name: Parameter name
        :param parameter61_value: Parameter value
        :param parameter62_name: Parameter name
        :param parameter62_value: Parameter value
        :param parameter63_name: Parameter name
        :param parameter63_value: Parameter value
        :param parameter64_name: Parameter name
        :param parameter64_value: Parameter value
        :param parameter65_name: Parameter name
        :param parameter65_value: Parameter value
        :param parameter66_name: Parameter name
        :param parameter66_value: Parameter value
        :param parameter67_name: Parameter name
        :param parameter67_value: Parameter value
        :param parameter68_name: Parameter name
        :param parameter68_value: Parameter value
        :param parameter69_name: Parameter name
        :param parameter69_value: Parameter value
        :param parameter70_name: Parameter name
        :param parameter70_value: Parameter value
        :param parameter71_name: Parameter name
        :param parameter71_value: Parameter value
        :param parameter72_name: Parameter name
        :param parameter72_value: Parameter value
        :param parameter73_name: Parameter name
        :param parameter73_value: Parameter value
        :param parameter74_name: Parameter name
        :param parameter74_value: Parameter value
        :param parameter75_name: Parameter name
        :param parameter75_value: Parameter value
        :param parameter76_name: Parameter name
        :param parameter76_value: Parameter value
        :param parameter77_name: Parameter name
        :param parameter77_value: Parameter value
        :param parameter78_name: Parameter name
        :param parameter78_value: Parameter value
        :param parameter79_name: Parameter name
        :param parameter79_value: Parameter value
        :param parameter80_name: Parameter name
        :param parameter80_value: Parameter value
        :param parameter81_name: Parameter name
        :param parameter81_value: Parameter value
        :param parameter82_name: Parameter name
        :param parameter82_value: Parameter value
        :param parameter83_name: Parameter name
        :param parameter83_value: Parameter value
        :param parameter84_name: Parameter name
        :param parameter84_value: Parameter value
        :param parameter85_name: Parameter name
        :param parameter85_value: Parameter value
        :param parameter86_name: Parameter name
        :param parameter86_value: Parameter value
        :param parameter87_name: Parameter name
        :param parameter87_value: Parameter value
        :param parameter88_name: Parameter name
        :param parameter88_value: Parameter value
        :param parameter89_name: Parameter name
        :param parameter89_value: Parameter value
        :param parameter90_name: Parameter name
        :param parameter90_value: Parameter value
        :param parameter91_name: Parameter name
        :param parameter91_value: Parameter value
        :param parameter92_name: Parameter name
        :param parameter92_value: Parameter value
        :param parameter93_name: Parameter name
        :param parameter93_value: Parameter value
        :param parameter94_name: Parameter name
        :param parameter94_value: Parameter value
        :param parameter95_name: Parameter name
        :param parameter95_value: Parameter value
        :param parameter96_name: Parameter name
        :param parameter96_value: Parameter value
        :param parameter97_name: Parameter name
        :param parameter97_value: Parameter value
        :param parameter98_name: Parameter name
        :param parameter98_value: Parameter value
        :param parameter99_name: Parameter name
        :param parameter99_value: Parameter value

        :returns: The created StreamInstance
        """

        data = values.of(
            {
                "Url": url,
                "Name": name,
                "Track": track,
                "StatusCallback": status_callback,
                "StatusCallbackMethod": status_callback_method,
                "Parameter1.Name": parameter1_name,
                "Parameter1.Value": parameter1_value,
                "Parameter2.Name": parameter2_name,
                "Parameter2.Value": parameter2_value,
                "Parameter3.Name": parameter3_name,
                "Parameter3.Value": parameter3_value,
                "Parameter4.Name": parameter4_name,
                "Parameter4.Value": parameter4_value,
                "Parameter5.Name": parameter5_name,
                "Parameter5.Value": parameter5_value,
                "Parameter6.Name": parameter6_name,
                "Parameter6.Value": parameter6_value,
                "Parameter7.Name": parameter7_name,
                "Parameter7.Value": parameter7_value,
                "Parameter8.Name": parameter8_name,
                "Parameter8.Value": parameter8_value,
                "Parameter9.Name": parameter9_name,
                "Parameter9.Value": parameter9_value,
                "Parameter10.Name": parameter10_name,
                "Parameter10.Value": parameter10_value,
                "Parameter11.Name": parameter11_name,
                "Parameter11.Value": parameter11_value,
                "Parameter12.Name": parameter12_name,
                "Parameter12.Value": parameter12_value,
                "Parameter13.Name": parameter13_name,
                "Parameter13.Value": parameter13_value,
                "Parameter14.Name": parameter14_name,
                "Parameter14.Value": parameter14_value,
                "Parameter15.Name": parameter15_name,
                "Parameter15.Value": parameter15_value,
                "Parameter16.Name": parameter16_name,
                "Parameter16.Value": parameter16_value,
                "Parameter17.Name": parameter17_name,
                "Parameter17.Value": parameter17_value,
                "Parameter18.Name": parameter18_name,
                "Parameter18.Value": parameter18_value,
                "Parameter19.Name": parameter19_name,
                "Parameter19.Value": parameter19_value,
                "Parameter20.Name": parameter20_name,
                "Parameter20.Value": parameter20_value,
                "Parameter21.Name": parameter21_name,
                "Parameter21.Value": parameter21_value,
                "Parameter22.Name": parameter22_name,
                "Parameter22.Value": parameter22_value,
                "Parameter23.Name": parameter23_name,
                "Parameter23.Value": parameter23_value,
                "Parameter24.Name": parameter24_name,
                "Parameter24.Value": parameter24_value,
                "Parameter25.Name": parameter25_name,
                "Parameter25.Value": parameter25_value,
                "Parameter26.Name": parameter26_name,
                "Parameter26.Value": parameter26_value,
                "Parameter27.Name": parameter27_name,
                "Parameter27.Value": parameter27_value,
                "Parameter28.Name": parameter28_name,
                "Parameter28.Value": parameter28_value,
                "Parameter29.Name": parameter29_name,
                "Parameter29.Value": parameter29_value,
                "Parameter30.Name": parameter30_name,
                "Parameter30.Value": parameter30_value,
                "Parameter31.Name": parameter31_name,
                "Parameter31.Value": parameter31_value,
                "Parameter32.Name": parameter32_name,
                "Parameter32.Value": parameter32_value,
                "Parameter33.Name": parameter33_name,
                "Parameter33.Value": parameter33_value,
                "Parameter34.Name": parameter34_name,
                "Parameter34.Value": parameter34_value,
                "Parameter35.Name": parameter35_name,
                "Parameter35.Value": parameter35_value,
                "Parameter36.Name": parameter36_name,
                "Parameter36.Value": parameter36_value,
                "Parameter37.Name": parameter37_name,
                "Parameter37.Value": parameter37_value,
                "Parameter38.Name": parameter38_name,
                "Parameter38.Value": parameter38_value,
                "Parameter39.Name": parameter39_name,
                "Parameter39.Value": parameter39_value,
                "Parameter40.Name": parameter40_name,
                "Parameter40.Value": parameter40_value,
                "Parameter41.Name": parameter41_name,
                "Parameter41.Value": parameter41_value,
                "Parameter42.Name": parameter42_name,
                "Parameter42.Value": parameter42_value,
                "Parameter43.Name": parameter43_name,
                "Parameter43.Value": parameter43_value,
                "Parameter44.Name": parameter44_name,
                "Parameter44.Value": parameter44_value,
                "Parameter45.Name": parameter45_name,
                "Parameter45.Value": parameter45_value,
                "Parameter46.Name": parameter46_name,
                "Parameter46.Value": parameter46_value,
                "Parameter47.Name": parameter47_name,
                "Parameter47.Value": parameter47_value,
                "Parameter48.Name": parameter48_name,
                "Parameter48.Value": parameter48_value,
                "Parameter49.Name": parameter49_name,
                "Parameter49.Value": parameter49_value,
                "Parameter50.Name": parameter50_name,
                "Parameter50.Value": parameter50_value,
                "Parameter51.Name": parameter51_name,
                "Parameter51.Value": parameter51_value,
                "Parameter52.Name": parameter52_name,
                "Parameter52.Value": parameter52_value,
                "Parameter53.Name": parameter53_name,
                "Parameter53.Value": parameter53_value,
                "Parameter54.Name": parameter54_name,
                "Parameter54.Value": parameter54_value,
                "Parameter55.Name": parameter55_name,
                "Parameter55.Value": parameter55_value,
                "Parameter56.Name": parameter56_name,
                "Parameter56.Value": parameter56_value,
                "Parameter57.Name": parameter57_name,
                "Parameter57.Value": parameter57_value,
                "Parameter58.Name": parameter58_name,
                "Parameter58.Value": parameter58_value,
                "Parameter59.Name": parameter59_name,
                "Parameter59.Value": parameter59_value,
                "Parameter60.Name": parameter60_name,
                "Parameter60.Value": parameter60_value,
                "Parameter61.Name": parameter61_name,
                "Parameter61.Value": parameter61_value,
                "Parameter62.Name": parameter62_name,
                "Parameter62.Value": parameter62_value,
                "Parameter63.Name": parameter63_name,
                "Parameter63.Value": parameter63_value,
                "Parameter64.Name": parameter64_name,
                "Parameter64.Value": parameter64_value,
                "Parameter65.Name": parameter65_name,
                "Parameter65.Value": parameter65_value,
                "Parameter66.Name": parameter66_name,
                "Parameter66.Value": parameter66_value,
                "Parameter67.Name": parameter67_name,
                "Parameter67.Value": parameter67_value,
                "Parameter68.Name": parameter68_name,
                "Parameter68.Value": parameter68_value,
                "Parameter69.Name": parameter69_name,
                "Parameter69.Value": parameter69_value,
                "Parameter70.Name": parameter70_name,
                "Parameter70.Value": parameter70_value,
                "Parameter71.Name": parameter71_name,
                "Parameter71.Value": parameter71_value,
                "Parameter72.Name": parameter72_name,
                "Parameter72.Value": parameter72_value,
                "Parameter73.Name": parameter73_name,
                "Parameter73.Value": parameter73_value,
                "Parameter74.Name": parameter74_name,
                "Parameter74.Value": parameter74_value,
                "Parameter75.Name": parameter75_name,
                "Parameter75.Value": parameter75_value,
                "Parameter76.Name": parameter76_name,
                "Parameter76.Value": parameter76_value,
                "Parameter77.Name": parameter77_name,
                "Parameter77.Value": parameter77_value,
                "Parameter78.Name": parameter78_name,
                "Parameter78.Value": parameter78_value,
                "Parameter79.Name": parameter79_name,
                "Parameter79.Value": parameter79_value,
                "Parameter80.Name": parameter80_name,
                "Parameter80.Value": parameter80_value,
                "Parameter81.Name": parameter81_name,
                "Parameter81.Value": parameter81_value,
                "Parameter82.Name": parameter82_name,
                "Parameter82.Value": parameter82_value,
                "Parameter83.Name": parameter83_name,
                "Parameter83.Value": parameter83_value,
                "Parameter84.Name": parameter84_name,
                "Parameter84.Value": parameter84_value,
                "Parameter85.Name": parameter85_name,
                "Parameter85.Value": parameter85_value,
                "Parameter86.Name": parameter86_name,
                "Parameter86.Value": parameter86_value,
                "Parameter87.Name": parameter87_name,
                "Parameter87.Value": parameter87_value,
                "Parameter88.Name": parameter88_name,
                "Parameter88.Value": parameter88_value,
                "Parameter89.Name": parameter89_name,
                "Parameter89.Value": parameter89_value,
                "Parameter90.Name": parameter90_name,
                "Parameter90.Value": parameter90_value,
                "Parameter91.Name": parameter91_name,
                "Parameter91.Value": parameter91_value,
                "Parameter92.Name": parameter92_name,
                "Parameter92.Value": parameter92_value,
                "Parameter93.Name": parameter93_name,
                "Parameter93.Value": parameter93_value,
                "Parameter94.Name": parameter94_name,
                "Parameter94.Value": parameter94_value,
                "Parameter95.Name": parameter95_name,
                "Parameter95.Value": parameter95_value,
                "Parameter96.Name": parameter96_name,
                "Parameter96.Value": parameter96_value,
                "Parameter97.Name": parameter97_name,
                "Parameter97.Value": parameter97_value,
                "Parameter98.Name": parameter98_name,
                "Parameter98.Value": parameter98_value,
                "Parameter99.Name": parameter99_name,
                "Parameter99.Value": parameter99_value,
            }
        )
        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = self._version.create(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            call_sid=self._solution["call_sid"],
        )

    async def create_async(
        self,
        url: str,
        name: Union[str, object] = values.unset,
        track: Union["StreamInstance.Track", object] = values.unset,
        status_callback: Union[str, object] = values.unset,
        status_callback_method: Union[str, object] = values.unset,
        parameter1_name: Union[str, object] = values.unset,
        parameter1_value: Union[str, object] = values.unset,
        parameter2_name: Union[str, object] = values.unset,
        parameter2_value: Union[str, object] = values.unset,
        parameter3_name: Union[str, object] = values.unset,
        parameter3_value: Union[str, object] = values.unset,
        parameter4_name: Union[str, object] = values.unset,
        parameter4_value: Union[str, object] = values.unset,
        parameter5_name: Union[str, object] = values.unset,
        parameter5_value: Union[str, object] = values.unset,
        parameter6_name: Union[str, object] = values.unset,
        parameter6_value: Union[str, object] = values.unset,
        parameter7_name: Union[str, object] = values.unset,
        parameter7_value: Union[str, object] = values.unset,
        parameter8_name: Union[str, object] = values.unset,
        parameter8_value: Union[str, object] = values.unset,
        parameter9_name: Union[str, object] = values.unset,
        parameter9_value: Union[str, object] = values.unset,
        parameter10_name: Union[str, object] = values.unset,
        parameter10_value: Union[str, object] = values.unset,
        parameter11_name: Union[str, object] = values.unset,
        parameter11_value: Union[str, object] = values.unset,
        parameter12_name: Union[str, object] = values.unset,
        parameter12_value: Union[str, object] = values.unset,
        parameter13_name: Union[str, object] = values.unset,
        parameter13_value: Union[str, object] = values.unset,
        parameter14_name: Union[str, object] = values.unset,
        parameter14_value: Union[str, object] = values.unset,
        parameter15_name: Union[str, object] = values.unset,
        parameter15_value: Union[str, object] = values.unset,
        parameter16_name: Union[str, object] = values.unset,
        parameter16_value: Union[str, object] = values.unset,
        parameter17_name: Union[str, object] = values.unset,
        parameter17_value: Union[str, object] = values.unset,
        parameter18_name: Union[str, object] = values.unset,
        parameter18_value: Union[str, object] = values.unset,
        parameter19_name: Union[str, object] = values.unset,
        parameter19_value: Union[str, object] = values.unset,
        parameter20_name: Union[str, object] = values.unset,
        parameter20_value: Union[str, object] = values.unset,
        parameter21_name: Union[str, object] = values.unset,
        parameter21_value: Union[str, object] = values.unset,
        parameter22_name: Union[str, object] = values.unset,
        parameter22_value: Union[str, object] = values.unset,
        parameter23_name: Union[str, object] = values.unset,
        parameter23_value: Union[str, object] = values.unset,
        parameter24_name: Union[str, object] = values.unset,
        parameter24_value: Union[str, object] = values.unset,
        parameter25_name: Union[str, object] = values.unset,
        parameter25_value: Union[str, object] = values.unset,
        parameter26_name: Union[str, object] = values.unset,
        parameter26_value: Union[str, object] = values.unset,
        parameter27_name: Union[str, object] = values.unset,
        parameter27_value: Union[str, object] = values.unset,
        parameter28_name: Union[str, object] = values.unset,
        parameter28_value: Union[str, object] = values.unset,
        parameter29_name: Union[str, object] = values.unset,
        parameter29_value: Union[str, object] = values.unset,
        parameter30_name: Union[str, object] = values.unset,
        parameter30_value: Union[str, object] = values.unset,
        parameter31_name: Union[str, object] = values.unset,
        parameter31_value: Union[str, object] = values.unset,
        parameter32_name: Union[str, object] = values.unset,
        parameter32_value: Union[str, object] = values.unset,
        parameter33_name: Union[str, object] = values.unset,
        parameter33_value: Union[str, object] = values.unset,
        parameter34_name: Union[str, object] = values.unset,
        parameter34_value: Union[str, object] = values.unset,
        parameter35_name: Union[str, object] = values.unset,
        parameter35_value: Union[str, object] = values.unset,
        parameter36_name: Union[str, object] = values.unset,
        parameter36_value: Union[str, object] = values.unset,
        parameter37_name: Union[str, object] = values.unset,
        parameter37_value: Union[str, object] = values.unset,
        parameter38_name: Union[str, object] = values.unset,
        parameter38_value: Union[str, object] = values.unset,
        parameter39_name: Union[str, object] = values.unset,
        parameter39_value: Union[str, object] = values.unset,
        parameter40_name: Union[str, object] = values.unset,
        parameter40_value: Union[str, object] = values.unset,
        parameter41_name: Union[str, object] = values.unset,
        parameter41_value: Union[str, object] = values.unset,
        parameter42_name: Union[str, object] = values.unset,
        parameter42_value: Union[str, object] = values.unset,
        parameter43_name: Union[str, object] = values.unset,
        parameter43_value: Union[str, object] = values.unset,
        parameter44_name: Union[str, object] = values.unset,
        parameter44_value: Union[str, object] = values.unset,
        parameter45_name: Union[str, object] = values.unset,
        parameter45_value: Union[str, object] = values.unset,
        parameter46_name: Union[str, object] = values.unset,
        parameter46_value: Union[str, object] = values.unset,
        parameter47_name: Union[str, object] = values.unset,
        parameter47_value: Union[str, object] = values.unset,
        parameter48_name: Union[str, object] = values.unset,
        parameter48_value: Union[str, object] = values.unset,
        parameter49_name: Union[str, object] = values.unset,
        parameter49_value: Union[str, object] = values.unset,
        parameter50_name: Union[str, object] = values.unset,
        parameter50_value: Union[str, object] = values.unset,
        parameter51_name: Union[str, object] = values.unset,
        parameter51_value: Union[str, object] = values.unset,
        parameter52_name: Union[str, object] = values.unset,
        parameter52_value: Union[str, object] = values.unset,
        parameter53_name: Union[str, object] = values.unset,
        parameter53_value: Union[str, object] = values.unset,
        parameter54_name: Union[str, object] = values.unset,
        parameter54_value: Union[str, object] = values.unset,
        parameter55_name: Union[str, object] = values.unset,
        parameter55_value: Union[str, object] = values.unset,
        parameter56_name: Union[str, object] = values.unset,
        parameter56_value: Union[str, object] = values.unset,
        parameter57_name: Union[str, object] = values.unset,
        parameter57_value: Union[str, object] = values.unset,
        parameter58_name: Union[str, object] = values.unset,
        parameter58_value: Union[str, object] = values.unset,
        parameter59_name: Union[str, object] = values.unset,
        parameter59_value: Union[str, object] = values.unset,
        parameter60_name: Union[str, object] = values.unset,
        parameter60_value: Union[str, object] = values.unset,
        parameter61_name: Union[str, object] = values.unset,
        parameter61_value: Union[str, object] = values.unset,
        parameter62_name: Union[str, object] = values.unset,
        parameter62_value: Union[str, object] = values.unset,
        parameter63_name: Union[str, object] = values.unset,
        parameter63_value: Union[str, object] = values.unset,
        parameter64_name: Union[str, object] = values.unset,
        parameter64_value: Union[str, object] = values.unset,
        parameter65_name: Union[str, object] = values.unset,
        parameter65_value: Union[str, object] = values.unset,
        parameter66_name: Union[str, object] = values.unset,
        parameter66_value: Union[str, object] = values.unset,
        parameter67_name: Union[str, object] = values.unset,
        parameter67_value: Union[str, object] = values.unset,
        parameter68_name: Union[str, object] = values.unset,
        parameter68_value: Union[str, object] = values.unset,
        parameter69_name: Union[str, object] = values.unset,
        parameter69_value: Union[str, object] = values.unset,
        parameter70_name: Union[str, object] = values.unset,
        parameter70_value: Union[str, object] = values.unset,
        parameter71_name: Union[str, object] = values.unset,
        parameter71_value: Union[str, object] = values.unset,
        parameter72_name: Union[str, object] = values.unset,
        parameter72_value: Union[str, object] = values.unset,
        parameter73_name: Union[str, object] = values.unset,
        parameter73_value: Union[str, object] = values.unset,
        parameter74_name: Union[str, object] = values.unset,
        parameter74_value: Union[str, object] = values.unset,
        parameter75_name: Union[str, object] = values.unset,
        parameter75_value: Union[str, object] = values.unset,
        parameter76_name: Union[str, object] = values.unset,
        parameter76_value: Union[str, object] = values.unset,
        parameter77_name: Union[str, object] = values.unset,
        parameter77_value: Union[str, object] = values.unset,
        parameter78_name: Union[str, object] = values.unset,
        parameter78_value: Union[str, object] = values.unset,
        parameter79_name: Union[str, object] = values.unset,
        parameter79_value: Union[str, object] = values.unset,
        parameter80_name: Union[str, object] = values.unset,
        parameter80_value: Union[str, object] = values.unset,
        parameter81_name: Union[str, object] = values.unset,
        parameter81_value: Union[str, object] = values.unset,
        parameter82_name: Union[str, object] = values.unset,
        parameter82_value: Union[str, object] = values.unset,
        parameter83_name: Union[str, object] = values.unset,
        parameter83_value: Union[str, object] = values.unset,
        parameter84_name: Union[str, object] = values.unset,
        parameter84_value: Union[str, object] = values.unset,
        parameter85_name: Union[str, object] = values.unset,
        parameter85_value: Union[str, object] = values.unset,
        parameter86_name: Union[str, object] = values.unset,
        parameter86_value: Union[str, object] = values.unset,
        parameter87_name: Union[str, object] = values.unset,
        parameter87_value: Union[str, object] = values.unset,
        parameter88_name: Union[str, object] = values.unset,
        parameter88_value: Union[str, object] = values.unset,
        parameter89_name: Union[str, object] = values.unset,
        parameter89_value: Union[str, object] = values.unset,
        parameter90_name: Union[str, object] = values.unset,
        parameter90_value: Union[str, object] = values.unset,
        parameter91_name: Union[str, object] = values.unset,
        parameter91_value: Union[str, object] = values.unset,
        parameter92_name: Union[str, object] = values.unset,
        parameter92_value: Union[str, object] = values.unset,
        parameter93_name: Union[str, object] = values.unset,
        parameter93_value: Union[str, object] = values.unset,
        parameter94_name: Union[str, object] = values.unset,
        parameter94_value: Union[str, object] = values.unset,
        parameter95_name: Union[str, object] = values.unset,
        parameter95_value: Union[str, object] = values.unset,
        parameter96_name: Union[str, object] = values.unset,
        parameter96_value: Union[str, object] = values.unset,
        parameter97_name: Union[str, object] = values.unset,
        parameter97_value: Union[str, object] = values.unset,
        parameter98_name: Union[str, object] = values.unset,
        parameter98_value: Union[str, object] = values.unset,
        parameter99_name: Union[str, object] = values.unset,
        parameter99_value: Union[str, object] = values.unset,
    ) -> StreamInstance:
        """
        Asynchronously create the StreamInstance

        :param url: Relative or absolute URL where WebSocket connection will be established.
        :param name: The user-specified name of this Stream, if one was given when the Stream was created. This can be used to stop the Stream.
        :param track:
        :param status_callback: Absolute URL to which Twilio sends status callback HTTP requests.
        :param status_callback_method: The HTTP method Twilio uses when sending `status_callback` requests. Possible values are `GET` and `POST`. Default is `POST`.
        :param parameter1_name: Parameter name
        :param parameter1_value: Parameter value
        :param parameter2_name: Parameter name
        :param parameter2_value: Parameter value
        :param parameter3_name: Parameter name
        :param parameter3_value: Parameter value
        :param parameter4_name: Parameter name
        :param parameter4_value: Parameter value
        :param parameter5_name: Parameter name
        :param parameter5_value: Parameter value
        :param parameter6_name: Parameter name
        :param parameter6_value: Parameter value
        :param parameter7_name: Parameter name
        :param parameter7_value: Parameter value
        :param parameter8_name: Parameter name
        :param parameter8_value: Parameter value
        :param parameter9_name: Parameter name
        :param parameter9_value: Parameter value
        :param parameter10_name: Parameter name
        :param parameter10_value: Parameter value
        :param parameter11_name: Parameter name
        :param parameter11_value: Parameter value
        :param parameter12_name: Parameter name
        :param parameter12_value: Parameter value
        :param parameter13_name: Parameter name
        :param parameter13_value: Parameter value
        :param parameter14_name: Parameter name
        :param parameter14_value: Parameter value
        :param parameter15_name: Parameter name
        :param parameter15_value: Parameter value
        :param parameter16_name: Parameter name
        :param parameter16_value: Parameter value
        :param parameter17_name: Parameter name
        :param parameter17_value: Parameter value
        :param parameter18_name: Parameter name
        :param parameter18_value: Parameter value
        :param parameter19_name: Parameter name
        :param parameter19_value: Parameter value
        :param parameter20_name: Parameter name
        :param parameter20_value: Parameter value
        :param parameter21_name: Parameter name
        :param parameter21_value: Parameter value
        :param parameter22_name: Parameter name
        :param parameter22_value: Parameter value
        :param parameter23_name: Parameter name
        :param parameter23_value: Parameter value
        :param parameter24_name: Parameter name
        :param parameter24_value: Parameter value
        :param parameter25_name: Parameter name
        :param parameter25_value: Parameter value
        :param parameter26_name: Parameter name
        :param parameter26_value: Parameter value
        :param parameter27_name: Parameter name
        :param parameter27_value: Parameter value
        :param parameter28_name: Parameter name
        :param parameter28_value: Parameter value
        :param parameter29_name: Parameter name
        :param parameter29_value: Parameter value
        :param parameter30_name: Parameter name
        :param parameter30_value: Parameter value
        :param parameter31_name: Parameter name
        :param parameter31_value: Parameter value
        :param parameter32_name: Parameter name
        :param parameter32_value: Parameter value
        :param parameter33_name: Parameter name
        :param parameter33_value: Parameter value
        :param parameter34_name: Parameter name
        :param parameter34_value: Parameter value
        :param parameter35_name: Parameter name
        :param parameter35_value: Parameter value
        :param parameter36_name: Parameter name
        :param parameter36_value: Parameter value
        :param parameter37_name: Parameter name
        :param parameter37_value: Parameter value
        :param parameter38_name: Parameter name
        :param parameter38_value: Parameter value
        :param parameter39_name: Parameter name
        :param parameter39_value: Parameter value
        :param parameter40_name: Parameter name
        :param parameter40_value: Parameter value
        :param parameter41_name: Parameter name
        :param parameter41_value: Parameter value
        :param parameter42_name: Parameter name
        :param parameter42_value: Parameter value
        :param parameter43_name: Parameter name
        :param parameter43_value: Parameter value
        :param parameter44_name: Parameter name
        :param parameter44_value: Parameter value
        :param parameter45_name: Parameter name
        :param parameter45_value: Parameter value
        :param parameter46_name: Parameter name
        :param parameter46_value: Parameter value
        :param parameter47_name: Parameter name
        :param parameter47_value: Parameter value
        :param parameter48_name: Parameter name
        :param parameter48_value: Parameter value
        :param parameter49_name: Parameter name
        :param parameter49_value: Parameter value
        :param parameter50_name: Parameter name
        :param parameter50_value: Parameter value
        :param parameter51_name: Parameter name
        :param parameter51_value: Parameter value
        :param parameter52_name: Parameter name
        :param parameter52_value: Parameter value
        :param parameter53_name: Parameter name
        :param parameter53_value: Parameter value
        :param parameter54_name: Parameter name
        :param parameter54_value: Parameter value
        :param parameter55_name: Parameter name
        :param parameter55_value: Parameter value
        :param parameter56_name: Parameter name
        :param parameter56_value: Parameter value
        :param parameter57_name: Parameter name
        :param parameter57_value: Parameter value
        :param parameter58_name: Parameter name
        :param parameter58_value: Parameter value
        :param parameter59_name: Parameter name
        :param parameter59_value: Parameter value
        :param parameter60_name: Parameter name
        :param parameter60_value: Parameter value
        :param parameter61_name: Parameter name
        :param parameter61_value: Parameter value
        :param parameter62_name: Parameter name
        :param parameter62_value: Parameter value
        :param parameter63_name: Parameter name
        :param parameter63_value: Parameter value
        :param parameter64_name: Parameter name
        :param parameter64_value: Parameter value
        :param parameter65_name: Parameter name
        :param parameter65_value: Parameter value
        :param parameter66_name: Parameter name
        :param parameter66_value: Parameter value
        :param parameter67_name: Parameter name
        :param parameter67_value: Parameter value
        :param parameter68_name: Parameter name
        :param parameter68_value: Parameter value
        :param parameter69_name: Parameter name
        :param parameter69_value: Parameter value
        :param parameter70_name: Parameter name
        :param parameter70_value: Parameter value
        :param parameter71_name: Parameter name
        :param parameter71_value: Parameter value
        :param parameter72_name: Parameter name
        :param parameter72_value: Parameter value
        :param parameter73_name: Parameter name
        :param parameter73_value: Parameter value
        :param parameter74_name: Parameter name
        :param parameter74_value: Parameter value
        :param parameter75_name: Parameter name
        :param parameter75_value: Parameter value
        :param parameter76_name: Parameter name
        :param parameter76_value: Parameter value
        :param parameter77_name: Parameter name
        :param parameter77_value: Parameter value
        :param parameter78_name: Parameter name
        :param parameter78_value: Parameter value
        :param parameter79_name: Parameter name
        :param parameter79_value: Parameter value
        :param parameter80_name: Parameter name
        :param parameter80_value: Parameter value
        :param parameter81_name: Parameter name
        :param parameter81_value: Parameter value
        :param parameter82_name: Parameter name
        :param parameter82_value: Parameter value
        :param parameter83_name: Parameter name
        :param parameter83_value: Parameter value
        :param parameter84_name: Parameter name
        :param parameter84_value: Parameter value
        :param parameter85_name: Parameter name
        :param parameter85_value: Parameter value
        :param parameter86_name: Parameter name
        :param parameter86_value: Parameter value
        :param parameter87_name: Parameter name
        :param parameter87_value: Parameter value
        :param parameter88_name: Parameter name
        :param parameter88_value: Parameter value
        :param parameter89_name: Parameter name
        :param parameter89_value: Parameter value
        :param parameter90_name: Parameter name
        :param parameter90_value: Parameter value
        :param parameter91_name: Parameter name
        :param parameter91_value: Parameter value
        :param parameter92_name: Parameter name
        :param parameter92_value: Parameter value
        :param parameter93_name: Parameter name
        :param parameter93_value: Parameter value
        :param parameter94_name: Parameter name
        :param parameter94_value: Parameter value
        :param parameter95_name: Parameter name
        :param parameter95_value: Parameter value
        :param parameter96_name: Parameter name
        :param parameter96_value: Parameter value
        :param parameter97_name: Parameter name
        :param parameter97_value: Parameter value
        :param parameter98_name: Parameter name
        :param parameter98_value: Parameter value
        :param parameter99_name: Parameter name
        :param parameter99_value: Parameter value

        :returns: The created StreamInstance
        """

        data = values.of(
            {
                "Url": url,
                "Name": name,
                "Track": track,
                "StatusCallback": status_callback,
                "StatusCallbackMethod": status_callback_method,
                "Parameter1.Name": parameter1_name,
                "Parameter1.Value": parameter1_value,
                "Parameter2.Name": parameter2_name,
                "Parameter2.Value": parameter2_value,
                "Parameter3.Name": parameter3_name,
                "Parameter3.Value": parameter3_value,
                "Parameter4.Name": parameter4_name,
                "Parameter4.Value": parameter4_value,
                "Parameter5.Name": parameter5_name,
                "Parameter5.Value": parameter5_value,
                "Parameter6.Name": parameter6_name,
                "Parameter6.Value": parameter6_value,
                "Parameter7.Name": parameter7_name,
                "Parameter7.Value": parameter7_value,
                "Parameter8.Name": parameter8_name,
                "Parameter8.Value": parameter8_value,
                "Parameter9.Name": parameter9_name,
                "Parameter9.Value": parameter9_value,
                "Parameter10.Name": parameter10_name,
                "Parameter10.Value": parameter10_value,
                "Parameter11.Name": parameter11_name,
                "Parameter11.Value": parameter11_value,
                "Parameter12.Name": parameter12_name,
                "Parameter12.Value": parameter12_value,
                "Parameter13.Name": parameter13_name,
                "Parameter13.Value": parameter13_value,
                "Parameter14.Name": parameter14_name,
                "Parameter14.Value": parameter14_value,
                "Parameter15.Name": parameter15_name,
                "Parameter15.Value": parameter15_value,
                "Parameter16.Name": parameter16_name,
                "Parameter16.Value": parameter16_value,
                "Parameter17.Name": parameter17_name,
                "Parameter17.Value": parameter17_value,
                "Parameter18.Name": parameter18_name,
                "Parameter18.Value": parameter18_value,
                "Parameter19.Name": parameter19_name,
                "Parameter19.Value": parameter19_value,
                "Parameter20.Name": parameter20_name,
                "Parameter20.Value": parameter20_value,
                "Parameter21.Name": parameter21_name,
                "Parameter21.Value": parameter21_value,
                "Parameter22.Name": parameter22_name,
                "Parameter22.Value": parameter22_value,
                "Parameter23.Name": parameter23_name,
                "Parameter23.Value": parameter23_value,
                "Parameter24.Name": parameter24_name,
                "Parameter24.Value": parameter24_value,
                "Parameter25.Name": parameter25_name,
                "Parameter25.Value": parameter25_value,
                "Parameter26.Name": parameter26_name,
                "Parameter26.Value": parameter26_value,
                "Parameter27.Name": parameter27_name,
                "Parameter27.Value": parameter27_value,
                "Parameter28.Name": parameter28_name,
                "Parameter28.Value": parameter28_value,
                "Parameter29.Name": parameter29_name,
                "Parameter29.Value": parameter29_value,
                "Parameter30.Name": parameter30_name,
                "Parameter30.Value": parameter30_value,
                "Parameter31.Name": parameter31_name,
                "Parameter31.Value": parameter31_value,
                "Parameter32.Name": parameter32_name,
                "Parameter32.Value": parameter32_value,
                "Parameter33.Name": parameter33_name,
                "Parameter33.Value": parameter33_value,
                "Parameter34.Name": parameter34_name,
                "Parameter34.Value": parameter34_value,
                "Parameter35.Name": parameter35_name,
                "Parameter35.Value": parameter35_value,
                "Parameter36.Name": parameter36_name,
                "Parameter36.Value": parameter36_value,
                "Parameter37.Name": parameter37_name,
                "Parameter37.Value": parameter37_value,
                "Parameter38.Name": parameter38_name,
                "Parameter38.Value": parameter38_value,
                "Parameter39.Name": parameter39_name,
                "Parameter39.Value": parameter39_value,
                "Parameter40.Name": parameter40_name,
                "Parameter40.Value": parameter40_value,
                "Parameter41.Name": parameter41_name,
                "Parameter41.Value": parameter41_value,
                "Parameter42.Name": parameter42_name,
                "Parameter42.Value": parameter42_value,
                "Parameter43.Name": parameter43_name,
                "Parameter43.Value": parameter43_value,
                "Parameter44.Name": parameter44_name,
                "Parameter44.Value": parameter44_value,
                "Parameter45.Name": parameter45_name,
                "Parameter45.Value": parameter45_value,
                "Parameter46.Name": parameter46_name,
                "Parameter46.Value": parameter46_value,
                "Parameter47.Name": parameter47_name,
                "Parameter47.Value": parameter47_value,
                "Parameter48.Name": parameter48_name,
                "Parameter48.Value": parameter48_value,
                "Parameter49.Name": parameter49_name,
                "Parameter49.Value": parameter49_value,
                "Parameter50.Name": parameter50_name,
                "Parameter50.Value": parameter50_value,
                "Parameter51.Name": parameter51_name,
                "Parameter51.Value": parameter51_value,
                "Parameter52.Name": parameter52_name,
                "Parameter52.Value": parameter52_value,
                "Parameter53.Name": parameter53_name,
                "Parameter53.Value": parameter53_value,
                "Parameter54.Name": parameter54_name,
                "Parameter54.Value": parameter54_value,
                "Parameter55.Name": parameter55_name,
                "Parameter55.Value": parameter55_value,
                "Parameter56.Name": parameter56_name,
                "Parameter56.Value": parameter56_value,
                "Parameter57.Name": parameter57_name,
                "Parameter57.Value": parameter57_value,
                "Parameter58.Name": parameter58_name,
                "Parameter58.Value": parameter58_value,
                "Parameter59.Name": parameter59_name,
                "Parameter59.Value": parameter59_value,
                "Parameter60.Name": parameter60_name,
                "Parameter60.Value": parameter60_value,
                "Parameter61.Name": parameter61_name,
                "Parameter61.Value": parameter61_value,
                "Parameter62.Name": parameter62_name,
                "Parameter62.Value": parameter62_value,
                "Parameter63.Name": parameter63_name,
                "Parameter63.Value": parameter63_value,
                "Parameter64.Name": parameter64_name,
                "Parameter64.Value": parameter64_value,
                "Parameter65.Name": parameter65_name,
                "Parameter65.Value": parameter65_value,
                "Parameter66.Name": parameter66_name,
                "Parameter66.Value": parameter66_value,
                "Parameter67.Name": parameter67_name,
                "Parameter67.Value": parameter67_value,
                "Parameter68.Name": parameter68_name,
                "Parameter68.Value": parameter68_value,
                "Parameter69.Name": parameter69_name,
                "Parameter69.Value": parameter69_value,
                "Parameter70.Name": parameter70_name,
                "Parameter70.Value": parameter70_value,
                "Parameter71.Name": parameter71_name,
                "Parameter71.Value": parameter71_value,
                "Parameter72.Name": parameter72_name,
                "Parameter72.Value": parameter72_value,
                "Parameter73.Name": parameter73_name,
                "Parameter73.Value": parameter73_value,
                "Parameter74.Name": parameter74_name,
                "Parameter74.Value": parameter74_value,
                "Parameter75.Name": parameter75_name,
                "Parameter75.Value": parameter75_value,
                "Parameter76.Name": parameter76_name,
                "Parameter76.Value": parameter76_value,
                "Parameter77.Name": parameter77_name,
                "Parameter77.Value": parameter77_value,
                "Parameter78.Name": parameter78_name,
                "Parameter78.Value": parameter78_value,
                "Parameter79.Name": parameter79_name,
                "Parameter79.Value": parameter79_value,
                "Parameter80.Name": parameter80_name,
                "Parameter80.Value": parameter80_value,
                "Parameter81.Name": parameter81_name,
                "Parameter81.Value": parameter81_value,
                "Parameter82.Name": parameter82_name,
                "Parameter82.Value": parameter82_value,
                "Parameter83.Name": parameter83_name,
                "Parameter83.Value": parameter83_value,
                "Parameter84.Name": parameter84_name,
                "Parameter84.Value": parameter84_value,
                "Parameter85.Name": parameter85_name,
                "Parameter85.Value": parameter85_value,
                "Parameter86.Name": parameter86_name,
                "Parameter86.Value": parameter86_value,
                "Parameter87.Name": parameter87_name,
                "Parameter87.Value": parameter87_value,
                "Parameter88.Name": parameter88_name,
                "Parameter88.Value": parameter88_value,
                "Parameter89.Name": parameter89_name,
                "Parameter89.Value": parameter89_value,
                "Parameter90.Name": parameter90_name,
                "Parameter90.Value": parameter90_value,
                "Parameter91.Name": parameter91_name,
                "Parameter91.Value": parameter91_value,
                "Parameter92.Name": parameter92_name,
                "Parameter92.Value": parameter92_value,
                "Parameter93.Name": parameter93_name,
                "Parameter93.Value": parameter93_value,
                "Parameter94.Name": parameter94_name,
                "Parameter94.Value": parameter94_value,
                "Parameter95.Name": parameter95_name,
                "Parameter95.Value": parameter95_value,
                "Parameter96.Name": parameter96_name,
                "Parameter96.Value": parameter96_value,
                "Parameter97.Name": parameter97_name,
                "Parameter97.Value": parameter97_value,
                "Parameter98.Name": parameter98_name,
                "Parameter98.Value": parameter98_value,
                "Parameter99.Name": parameter99_name,
                "Parameter99.Value": parameter99_value,
            }
        )
        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = await self._version.create_async(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return StreamInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            call_sid=self._solution["call_sid"],
        )

    def get(self, sid: str) -> StreamContext:
        """
        Constructs a StreamContext

        :param sid: The SID or the `name` of the Stream resource to be stopped
        """
        return StreamContext(
            self._version,
            account_sid=self._solution["account_sid"],
            call_sid=self._solution["call_sid"],
            sid=sid,
        )

    def __call__(self, sid: str) -> StreamContext:
        """
        Constructs a StreamContext

        :param sid: The SID or the `name` of the Stream resource to be stopped
        """
        return StreamContext(
            self._version,
            account_sid=self._solution["account_sid"],
            call_sid=self._solution["call_sid"],
            sid=sid,
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Api.V2010.StreamList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Verify
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import datetime
from typing import Any, Dict, Optional, Union
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class AccessTokenInstance(InstanceResource):

    class FactorTypes(object):
        PUSH = "push"

    """
    :ivar sid: A 34 character string that uniquely identifies this Access Token.
    :ivar account_sid: The unique SID identifier of the Account.
    :ivar service_sid: The unique SID identifier of the Verify Service.
    :ivar entity_identity: The unique external identifier for the Entity of the Service.
    :ivar factor_type: 
    :ivar factor_friendly_name: A human readable description of this factor, up to 64 characters. For a push factor, this can be the device's name.
    :ivar token: The access token generated for enrollment, this is an encrypted json web token.
    :ivar url: The URL of this resource.
    :ivar ttl: How long, in seconds, the access token is valid. Max: 5 minutes
    :ivar date_created: The date that this access token was created, given in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        service_sid: str,
        sid: Optional[str] = None,
    ):
        super().__init__(version)

        self.sid: Optional[str] = payload.get("sid")
        self.account_sid: Optional[str] = payload.get("account_sid")
        self.service_sid: Optional[str] = payload.get("service_sid")
        self.entity_identity: Optional[str] = payload.get("entity_identity")
        self.factor_type: Optional["AccessTokenInstance.FactorTypes"] = payload.get(
            "factor_type"
        )
        self.factor_friendly_name: Optional[str] = payload.get("factor_friendly_name")
        self.token: Optional[str] = payload.get("token")
        self.url: Optional[str] = payload.get("url")
        self.ttl: Optional[int] = deserialize.integer(payload.get("ttl"))
        self.date_created: Optional[datetime] = deserialize.iso8601_datetime(
            payload.get("date_created")
        )

        self._solution = {
            "service_sid": service_sid,
            "sid": sid or self.sid,
        }
        self._context: Optional[AccessTokenContext] = None

    @property
    def _proxy(self) -> "AccessTokenContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: AccessTokenContext for this AccessTokenInstance
        """
        if self._context is None:
            self._context = AccessTokenContext(
                self._version,
                service_sid=self._solution["service_sid"],
                sid=self._solution["sid"],
            )
        return self._context

    def fetch(self) -> "AccessTokenInstance":
        """
        Fetch the AccessTokenInstance


        :returns: The fetched AccessTokenInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "AccessTokenInstance":
        """
        Asynchronous coroutine to fetch the AccessTokenInstance


        :returns: The fetched AccessTokenInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Verify.V2.AccessTokenInstance {}>".format(context)


class AccessTokenContext(InstanceContext):

    def __init__(self, version: Version, service_sid: str, sid: str):
        """
        Initialize the AccessTokenContext

        :param version: Version that contains the resource
        :param service_sid: The unique SID identifier of the Service.
        :param sid: A 34 character string that uniquely identifies this Access Token.
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "service_sid": service_sid,
            "sid": sid,
        }
        self._uri = "/Services/{service_sid}/AccessTokens/{sid}".format(
            **self._solution
        )

    def fetch(self) -> AccessTokenInstance:
        """
        Fetch the AccessTokenInstance


        :returns: The fetched AccessTokenInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = self._version.fetch(method="GET", uri=self._uri, headers=headers)

        return AccessTokenInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            sid=self._solution["sid"],
        )

    async def fetch_async(self) -> AccessTokenInstance:
        """
        Asynchronous coroutine to fetch the AccessTokenInstance


        :returns: The fetched AccessTokenInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = await self._version.fetch_async(
            method="GET", uri=self._uri, headers=headers
        )

        return AccessTokenInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            sid=self._solution["sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Verify.V2.AccessTokenContext {}>".format(context)


class AccessTokenList(ListResource):

    def __init__(self, version: Version, service_sid: str):
        """
        Initialize the AccessTokenList

        :param version: Version that contains the resource
        :param service_sid: The unique SID identifier of the Service.

        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "service_sid": service_sid,
        }
        self._uri = "/Services/{service_sid}/AccessTokens".format(**self._solution)

    def create(
        self,
        identity: str,
        factor_type: "AccessTokenInstance.FactorTypes",
        factor_friendly_name: Union[str, object] = values.unset,
        ttl: Union[int, object] = values.unset,
    ) -> AccessTokenInstance:
        """
        Create the AccessTokenInstance

        :param identity: The unique external identifier for the Entity of the Service. This identifier should be immutable, not PII, and generated by your external system, such as your user's UUID, GUID, or SID.
        :param factor_type:
        :param factor_friendly_name: The friendly name of the factor that is going to be created with this access token
        :param ttl: How long, in seconds, the access token is valid. Can be an integer between 60 and 300. Default is 60.

        :returns: The created AccessTokenInstance
        """

        data = values.of(
            {
                "Identity": identity,
                "FactorType": factor_type,
                "FactorFriendlyName": factor_friendly_name,
                "Ttl": ttl,
            }
        )
        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = self._version.create(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return AccessTokenInstance(
            self._version, payload, service_sid=self._solution["service_sid"]
        )

    async def create_async(
        self,
        identity: str,
        factor_type: "AccessTokenInstance.FactorTypes",
        factor_friendly_name: Union[str, object] = values.unset,
        ttl: Union[int, object] = values.unset,
    ) -> AccessTokenInstance:
        """
        Asynchronously create the AccessTokenInstance

        :param identity: The unique external identifier for the Entity of the Service. This identifier should be immutable, not PII, and generated by your external system, such as your user's UUID, GUID, or SID.
        :param factor_type:
        :param factor_friendly_name: The friendly name of the factor that is going to be created with this access token
        :param ttl: How long, in seconds, the access token is valid. Can be an integer between 60 and 300. Default is 60.

        :returns: The created AccessTokenInstance
        """

        data = values.of(
            {
                "Identity": identity,
                "FactorType": factor_type,
                "FactorFriendlyName": factor_friendly_name,
                "Ttl": ttl,
            }
        )
        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = await self._version.create_async(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return AccessTokenInstance(
            self._version, payload, service_sid=self._solution["service_sid"]
        )

    def get(self, sid: str) -> AccessTokenContext:
        """
        Constructs a AccessTokenContext

        :param sid: A 34 character string that uniquely identifies this Access Token.
        """
        return AccessTokenContext(
            self._version, service_sid=self._solution["service_sid"], sid=sid
        )

    def __call__(self, sid: str) -> AccessTokenContext:
        """
        Constructs a AccessTokenContext

        :param sid: A 34 character string that uniquely identifies this Access Token.
        """
        return AccessTokenContext(
            self._version, service_sid=self._solution["service_sid"], sid=sid
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Verify.V2.AccessTokenList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Conversations
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import datetime
from typing import Any, Dict, List, Optional, Union, Iterator, AsyncIterator
from twilio.base import deserialize, serialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class BindingInstance(InstanceResource):

    class BindingType(object):
        APN = "apn"
        GCM = "gcm"
        FCM = "fcm"

    """
    :ivar sid: A 34 character string that uniquely identifies this resource.
    :ivar account_sid: The unique ID of the [Account](https://www.twilio.com/docs/iam/api/account) responsible for this binding.
    :ivar chat_service_sid: The SID of the [Conversation Service](https://www.twilio.com/docs/conversations/api/service-resource) the Binding resource is associated with.
    :ivar credential_sid: The SID of the [Credential](https://www.twilio.com/docs/conversations/api/credential-resource) for the binding. See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
    :ivar date_created: The date that this resource was created.
    :ivar date_updated: The date that this resource was last updated.
    :ivar endpoint: The unique endpoint identifier for the Binding. The format of this value depends on the `binding_type`.
    :ivar identity: The application-defined string that uniquely identifies the [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) within the [Conversation Service](https://www.twilio.com/docs/conversations/api/service-resource). See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more info.
    :ivar binding_type: 
    :ivar message_types: The [Conversation message types](https://www.twilio.com/docs/chat/push-notification-configuration#push-types) the binding is subscribed to.
    :ivar url: An absolute API resource URL for this binding.
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        chat_service_sid: str,
        sid: Optional[str] = None,
    ):
        super().__init__(version)

        self.sid: Optional[str] = payload.get("sid")
        self.account_sid: Optional[str] = payload.get("account_sid")
        self.chat_service_sid: Optional[str] = payload.get("chat_service_sid")
        self.credential_sid: Optional[str] = payload.get("credential_sid")
        self.date_created: Optional[datetime] = deserialize.iso8601_datetime(
            payload.get("date_created")
        )
        self.date_updated: Optional[datetime] = deserialize.iso8601_datetime(
            payload.get("date_updated")
        )
        self.endpoint: Optional[str] = payload.get("endpoint")
        self.identity: Optional[str] = payload.get("identity")
        self.binding_type: Optional["BindingInstance.BindingType"] = payload.get(
            "binding_type"
        )
        self.message_types: Optional[List[str]] = payload.get("message_types")
        self.url: Optional[str] = payload.get("url")

        self._solution = {
            "chat_service_sid": chat_service_sid,
            "sid": sid or self.sid,
        }
        self._context: Optional[BindingContext] = None

    @property
    def _proxy(self) -> "BindingContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: BindingContext for this BindingInstance
        """
        if self._context is None:
            self._context = BindingContext(
                self._version,
                chat_service_sid=self._solution["chat_service_sid"],
                sid=self._solution["sid"],
            )
        return self._context

    def delete(self) -> bool:
        """
        Deletes the BindingInstance


        :returns: True if delete succeeds, False otherwise
        """
        return self._proxy.delete()

    async def delete_async(self) -> bool:
        """
        Asynchronous coroutine that deletes the BindingInstance


        :returns: True if delete succeeds, False otherwise
        """
        return await self._proxy.delete_async()

    def fetch(self) -> "BindingInstance":
        """
        Fetch the BindingInstance


        :returns: The fetched BindingInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "BindingInstance":
        """
        Asynchronous coroutine to fetch the BindingInstance


        :returns: The fetched BindingInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Conversations.V1.BindingInstance {}>".format(context)


class BindingContext(InstanceContext):

    def __init__(self, version: Version, chat_service_sid: str, sid: str):
        """
        Initialize the BindingContext

        :param version: Version that contains the resource
        :param chat_service_sid: The SID of the [Conversation Service](https://www.twilio.com/docs/conversations/api/service-resource) the Binding resource is associated with.
        :param sid: A 34 character string that uniquely identifies this resource.
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "chat_service_sid": chat_service_sid,
            "sid": sid,
        }
        self._uri = "/Services/{chat_service_sid}/Bindings/{sid}".format(
            **self._solution
        )

    def delete(self) -> bool:
        """
        Deletes the BindingInstance


        :returns: True if delete succeeds, False otherwise
        """

        headers = values.of({})

        return self._version.delete(method="DELETE", uri=self._uri, headers=headers)

    async def delete_async(self) -> bool:
        """
        Asynchronous coroutine that deletes the BindingInstance


        :returns: True if delete succeeds, False otherwise
        """

        headers = values.of({})

        return await self._version.delete_async(
            method="DELETE", uri=self._uri, headers=headers
        )

    def fetch(self) -> BindingInstance:
        """
        Fetch the BindingInstance


        :returns: The fetched BindingInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = self._version.fetch(method="GET", uri=self._uri, headers=headers)

        return BindingInstance(
            self._version,
            payload,
            chat_service_sid=self._solution["chat_service_sid"],
            sid=self._solution["sid"],
        )

    async def fetch_async(self) -> BindingInstance:
        """
        Asynchronous coroutine to fetch the BindingInstance


        :returns: The fetched BindingInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = await self._version.fetch_async(
            method="GET", uri=self._uri, headers=headers
        )

        return BindingInstance(
            self._version,
            payload,
            chat_service_sid=self._solution["chat_service_sid"],
            sid=self._solution["sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Conversations.V1.BindingContext {}>".format(context)


class BindingPage(Page):

    def get_instance(self, payload: Dict[str, Any]) -> BindingInstance:
        """
        Build an instance of BindingInstance

        :param payload: Payload response from the API
        """
        return BindingInstance(
            self._version, payload, chat_service_sid=self._solution["chat_service_sid"]
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Conversations.V1.BindingPage>"


class BindingList(ListResource):

    def __init__(self, version: Version, chat_service_sid: str):
        """
        Initialize the BindingList

        :param version: Version that contains the resource
        :param chat_service_sid: The SID of the [Conversation Service](https://www.twilio.com/docs/conversations/api/service-resource) the Binding resource is associated with.

        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "chat_service_sid": chat_service_sid,
        }
        self._uri = "/Services/{chat_service_sid}/Bindings".format(**self._solution)

    def stream(
        self,
        binding_type: Union[List["BindingInstance.BindingType"], object] = values.unset,
        identity: Union[List[str], object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> Iterator[BindingInstance]:
        """
        Streams BindingInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param List[&quot;BindingInstance.BindingType&quot;] binding_type: The push technology used by the Binding resources to read.  Can be: `apn`, `gcm`, or `fcm`.  See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
        :param List[str] identity: The identity of a [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) this binding belongs to. See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more details.
        :param limit: Upper limit for the number of records to return. stream()
                      guarantees to never return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, stream() will attempt to read the
                          limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(
            binding_type=binding_type, identity=identity, page_size=limits["page_size"]
        )

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self,
        binding_type: Union[List["BindingInstance.BindingType"], object] = values.unset,
        identity: Union[List[str], object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> AsyncIterator[BindingInstance]:
        """
        Asynchronously streams BindingInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param List[&quot;BindingInstance.BindingType&quot;] binding_type: The push technology used by the Binding resources to read.  Can be: `apn`, `gcm`, or `fcm`.  See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
        :param List[str] identity: The identity of a [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) this binding belongs to. See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more details.
        :param limit: Upper limit for the number of records to return. stream()
                      guarantees to never return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, stream() will attempt to read the
                          limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            binding_type=binding_type, identity=identity, page_size=limits["page_size"]
        )

        return self._version.stream_async(page, limits["limit"])

    def list(
        self,
        binding_type: Union[List["BindingInstance.BindingType"], object] = values.unset,
        identity: Union[List[str], object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[BindingInstance]:
        """
        Lists BindingInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param List[&quot;BindingInstance.BindingType&quot;] binding_type: The push technology used by the Binding resources to read.  Can be: `apn`, `gcm`, or `fcm`.  See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
        :param List[str] identity: The identity of a [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) this binding belongs to. See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more details.
        :param limit: Upper limit for the number of records to return. list() guarantees
                      never to return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, list() will attempt to read the limit
                          with the most efficient page size, i.e. min(limit, 1000)

        :returns: list that will contain up to limit results
        """
        return list(
            self.stream(
                binding_type=binding_type,
                identity=identity,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self,
        binding_type: Union[List["BindingInstance.BindingType"], object] = values.unset,
        identity: Union[List[str], object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[BindingInstance]:
        """
        Asynchronously lists BindingInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param List[&quot;BindingInstance.BindingType&quot;] binding_type: The push technology used by the Binding resources to read.  Can be: `apn`, `gcm`, or `fcm`.  See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
        :param List[str] identity: The identity of a [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) this binding belongs to. See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more details.
        :param limit: Upper limit for the number of records to return. list() guarantees
                      never to return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, list() will attempt to read the limit
                          with the most efficient page size, i.e. min(limit, 1000)

        :returns: list that will contain up to limit results
        """
        return [
            record
            async for record in await self.stream_async(
                binding_type=binding_type,
                identity=identity,
                limit=limit,
                page_size=page_size,
            )
        ]

    def page(
        self,
        binding_type: Union[List["BindingInstance.BindingType"], object] = values.unset,
        identity: Union[List[str], object] = values.unset,
        page_token: Union[str, object] = values.unset,
        page_number: Union[int, object] = values.unset,
        page_size: Union[int, object] = values.unset,
    ) -> BindingPage:
        """
        Retrieve a single page of BindingInstance records from the API.
        Request is executed immediately

        :param binding_type: The push technology used by the Binding resources to read.  Can be: `apn`, `gcm`, or `fcm`.  See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
        :param identity: The identity of a [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) this binding belongs to. See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more details.
        :param page_token: PageToken provided by the API
        :param page_number: Page Number, this value is simply for client state
        :param page_size: Number of records to return, defaults to 50

        :returns: Page of BindingInstance
        """
        data = values.of(
            {
                "BindingType": serialize.map(binding_type, lambda e: e),
                "Identity": serialize.map(identity, lambda e: e),
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Accept"] = "application/json"

        response = self._version.page(
            method="GET", uri=self._uri, params=data, headers=headers
        )
        return BindingPage(self._version, response, self._solution)

    async def page_async(
        self,
        binding_type: Union[List["BindingInstance.BindingType"], object] = values.unset,
        identity: Union[List[str], object] = values.unset,
        page_token: Union[str, object] = values.unset,
        page_number: Union[int, object] = values.unset,
        page_size: Union[int, object] = values.unset,
    ) -> BindingPage:
        """
        Asynchronously retrieve a single page of BindingInstance records from the API.
        Request is executed immediately

        :param binding_type: The push technology used by the Binding resources to read.  Can be: `apn`, `gcm`, or `fcm`.  See [push notification configuration](https://www.twilio.com/docs/chat/push-notification-configuration) for more info.
        :param identity: The identity of a [Conversation User](https://www.twilio.com/docs/conversations/api/user-resource) this binding belongs to. See [access tokens](https://www.twilio.com/docs/conversations/create-tokens) for more details.
        :param page_token: PageToken provided by the API
        :param page_number: Page Number, this value is simply for client state
        :param page_size: Number of records to return, defaults to 50

        :returns: Page of BindingInstance
        """
        data = values.of(
            {
                "BindingType": serialize.map(binding_type, lambda e: e),
                "Identity": serialize.map(identity, lambda e: e),
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Accept"] = "application/json"

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data, headers=headers
        )
        return BindingPage(self._version, response, self._solution)

    def get_page(self, target_url: str) -> BindingPage:
        """
        Retrieve a specific page of BindingInstance records from the API.
        Request is executed immediately

        :param target_url: API-generated URL for the requested results page

        :returns: Page of BindingInstance
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return BindingPage(self._version, response, self._solution)

    async def get_page_async(self, target_url: str) -> BindingPage:
        """
        Asynchronously retrieve a specific page of BindingInstance records from the API.
        Request is executed immediately

        :param target_url: API-generated URL for the requested results page

        :returns: Page of BindingInstance
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return BindingPage(self._version, response, self._solution)

    def get(self, sid: str) -> BindingContext:
        """
        Constructs a BindingContext

        :param sid: A 34 character string that uniquely identifies this resource.
        """
        return BindingContext(
            self._version, chat_service_sid=self._solution["chat_service_sid"], sid=sid
        )

    def __call__(self, sid: str) -> BindingContext:
        """
        Constructs a BindingContext

        :param sid: A 34 character string that uniquely identifies this resource.
        """
        return BindingContext(
            self._version, chat_service_sid=self._solution["chat_service_sid"], sid=sid
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Conversations.V1.BindingList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Bulkexports
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from typing import Any, Dict, Optional, Union
from twilio.base import serialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class ExportConfigurationInstance(InstanceResource):
    """
    :ivar enabled: If true, Twilio will automatically generate every day's file when the day is over.
    :ivar webhook_url: Stores the URL destination for the method specified in webhook_method.
    :ivar webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url
    :ivar resource_type: The type of communication – Messages, Calls, Conferences, and Participants
    :ivar url: The URL of this resource.
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        resource_type: Optional[str] = None,
    ):
        super().__init__(version)

        self.enabled: Optional[bool] = payload.get("enabled")
        self.webhook_url: Optional[str] = payload.get("webhook_url")
        self.webhook_method: Optional[str] = payload.get("webhook_method")
        self.resource_type: Optional[str] = payload.get("resource_type")
        self.url: Optional[str] = payload.get("url")

        self._solution = {
            "resource_type": resource_type or self.resource_type,
        }
        self._context: Optional[ExportConfigurationContext] = None

    @property
    def _proxy(self) -> "ExportConfigurationContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: ExportConfigurationContext for this ExportConfigurationInstance
        """
        if self._context is None:
            self._context = ExportConfigurationContext(
                self._version,
                resource_type=self._solution["resource_type"],
            )
        return self._context

    def fetch(self) -> "ExportConfigurationInstance":
        """
        Fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "ExportConfigurationInstance":
        """
        Asynchronous coroutine to fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        enabled: Union[bool, object] = values.unset,
        webhook_url: Union[str, object] = values.unset,
        webhook_method: Union[str, object] = values.unset,
    ) -> "ExportConfigurationInstance":
        """
        Update the ExportConfigurationInstance

        :param enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        """
        return self._proxy.update(
            enabled=enabled,
            webhook_url=webhook_url,
            webhook_method=webhook_method,
        )

    async def update_async(
        self,
        enabled: Union[bool, object] = values.unset,
        webhook_url: Union[str, object] = values.unset,
        webhook_method: Union[str, object] = values.unset,
    ) -> "ExportConfigurationInstance":
        """
        Asynchronous coroutine to update the ExportConfigurationInstance

        :param enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        """
        return await self._proxy.update_async(
            enabled=enabled,
            webhook_url=webhook_url,
            webhook_method=webhook_method,
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Bulkexports.V1.ExportConfigurationInstance {}>".format(context)


class ExportConfigurationContext(InstanceContext):

    def __init__(self, version: Version, resource_type: str):
        """
        Initialize the ExportConfigurationContext

        :param version: Version that contains the resource
        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "resource_type": resource_type,
        }
        self._uri = "/Exports/{resource_type}/Configuration".format(**self._solution)

    def fetch(self) -> ExportConfigurationInstance:
        """
        Fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = self._version.fetch(method="GET", uri=self._uri, headers=headers)

        return ExportConfigurationInstance(
            self._version,
            payload,
            resource_type=self._solution["resource_type"],
        )

    async def fetch_async(self) -> ExportConfigurationInstance:
        """
        Asynchronous coroutine to fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = await self._version.fetch_async(
            method="GET", uri=self._uri, headers=headers
        )

        return ExportConfigurationInstance(
            self._version,
            payload,
            resource_type=self._solution["resource_type"],
        )

    def update(
        self,
        enabled: Union[bool, object] = values.unset,
        webhook_url: Union[str, object] = values.unset,
        webhook_method: Union[str, object] = values.unset,
    ) -> ExportConfigurationInstance:
        """
        Update the ExportConfigurationInstance

        :param enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        """

        data = values.of(
            {
                "Enabled": serialize.boolean_to_string(enabled),
                "WebhookUrl": webhook_url,
                "WebhookMethod": webhook_method,
            }
        )
        headers = values.of({})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = self._version.update(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return ExportConfigurationInstance(
            self._version, payload, resource_type=self._solution["resource_type"]
        )

    async def update_async(
        self,
        enabled: Union[bool, object] = values.unset,
        webhook_url: Union[str, object] = values.unset,
        webhook_method: Union[str, object] = values.unset,
    ) -> ExportConfigurationInstance:
        """
        Asynchronous coroutine to update the ExportConfigurationInstance

        :param enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        """

        data = values.of(
            {
                "Enabled": serialize.boolean_to_string(enabled),
                "WebhookUrl": webhook_url,
                "WebhookMethod": webhook_method,
            }
        )
        headers = values.of({})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = await self._version.update_async(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return ExportConfigurationInstance(
            self._version, payload, resource_type=self._solution["resource_type"]
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Bulkexports.V1.ExportConfigurationContext {}>".format(context)


class ExportConfigurationList(ListResource):

    def __init__(self, version: Version):
        """
        Initialize the ExportConfigurationList

        :param version: Version that contains the resource

        """
        super().__init__(version)

    def get(self, resource_type: str) -> ExportConfigurationContext:
        """
        Constructs a ExportConfigurationContext

        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants
        """
        return ExportConfigurationContext(self._version, resource_type=resource_type)

    def __call__(self, resource_type: str) -> ExportConfigurationContext:
        """
        Constructs a ExportConfigurationContext

        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants
        """
        return ExportConfigurationContext(self._version, resource_type=resource_type)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Bulkexports.V1.ExportConfigurationList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Taskrouter
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import datetime
from typing import Any, Dict, List, Optional, Union, Iterator, AsyncIterator
from twilio.base import deserialize, serialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page
from twilio.rest.taskrouter.v1.workspace.activity import ActivityList
from twilio.rest.taskrouter.v1.workspace.event import EventList
from twilio.rest.taskrouter.v1.workspace.task import TaskList
from twilio.rest.taskrouter.v1.workspace.task_channel import TaskChannelList
from twilio.rest.taskrouter.v1.workspace.task_queue import TaskQueueList
from twilio.rest.taskrouter.v1.workspace.worker import WorkerList
from twilio.rest.taskrouter.v1.workspace.workflow import WorkflowList
from twilio.rest.taskrouter.v1.workspace.workspace_cumulative_statistics import (
    WorkspaceCumulativeStatisticsList,
)
from twilio.rest.taskrouter.v1.workspace.workspace_real_time_statistics import (
    WorkspaceRealTimeStatisticsList,
)
from twilio.rest.taskrouter.v1.workspace.workspace_statistics import (
    WorkspaceStatisticsList,
)


class WorkspaceInstance(InstanceResource):

    class QueueOrder(object):
        FIFO = "FIFO"
        LIFO = "LIFO"

    """
    :ivar account_sid: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the Workspace resource.
    :ivar date_created: The date and time in GMT when the resource was created specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
    :ivar date_updated: The date and time in GMT when the resource was last updated specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
    :ivar default_activity_name: The name of the default activity.
    :ivar default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
    :ivar event_callback_url: The URL we call when an event occurs. If provided, the Workspace will publish events to this URL, for example, to collect data for reporting. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
    :ivar events_filter: The list of Workspace events for which to call `event_callback_url`. For example, if `EventsFilter=task.created, task.canceled, worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
    :ivar friendly_name: The string that you assigned to describe the Workspace resource. For example `Customer Support` or `2014 Election Campaign`.
    :ivar multi_task_enabled: Whether multi-tasking is enabled. The default is `true`, which enables multi-tasking. Multi-tasking allows Workers to handle multiple Tasks simultaneously. When enabled (`true`), each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking each Worker would only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
    :ivar sid: The unique string that we created to identify the Workspace resource.
    :ivar timeout_activity_name: The name of the timeout activity.
    :ivar timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
    :ivar prioritize_queue_order: 
    :ivar url: The absolute URL of the Workspace resource.
    :ivar links: The URLs of related resources.
    """

    def __init__(
        self, version: Version, payload: Dict[str, Any], sid: Optional[str] = None
    ):
        super().__init__(version)

        self.account_sid: Optional[str] = payload.get("account_sid")
        self.date_created: Optional[datetime] = deserialize.iso8601_datetime(
            payload.get("date_created")
        )
        self.date_updated: Optional[datetime] = deserialize.iso8601_datetime(
            payload.get("date_updated")
        )
        self.default_activity_name: Optional[str] = payload.get("default_activity_name")
        self.default_activity_sid: Optional[str] = payload.get("default_activity_sid")
        self.event_callback_url: Optional[str] = payload.get("event_callback_url")
        self.events_filter: Optional[str] = payload.get("events_filter")
        self.friendly_name: Optional[str] = payload.get("friendly_name")
        self.multi_task_enabled: Optional[bool] = payload.get("multi_task_enabled")
        self.sid: Optional[str] = payload.get("sid")
        self.timeout_activity_name: Optional[str] = payload.get("timeout_activity_name")
        self.timeout_activity_sid: Optional[str] = payload.get("timeout_activity_sid")
        self.prioritize_queue_order: Optional["WorkspaceInstance.QueueOrder"] = (
            payload.get("prioritize_queue_order")
        )
        self.url: Optional[str] = payload.get("url")
        self.links: Optional[Dict[str, object]] = payload.get("links")

        self._solution = {
            "sid": sid or self.sid,
        }
        self._context: Optional[WorkspaceContext] = None

    @property
    def _proxy(self) -> "WorkspaceContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: WorkspaceContext for this WorkspaceInstance
        """
        if self._context is None:
            self._context = WorkspaceContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    def delete(self) -> bool:
        """
        Deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        """
        return self._proxy.delete()

    async def delete_async(self) -> bool:
        """
        Asynchronous coroutine that deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        """
        return await self._proxy.delete_async()

    def fetch(self) -> "WorkspaceInstance":
        """
        Fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "WorkspaceInstance":
        """
        Asynchronous coroutine to fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        default_activity_sid: Union[str, object] = values.unset,
        event_callback_url: Union[str, object] = values.unset,
        events_filter: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        multi_task_enabled: Union[bool, object] = values.unset,
        timeout_activity_sid: Union[str, object] = values.unset,
        prioritize_queue_order: Union[
            "WorkspaceInstance.QueueOrder", object
        ] = values.unset,
    ) -> "WorkspaceInstance":
        """
        Update the WorkspaceInstance

        :param default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        """
        return self._proxy.update(
            default_activity_sid=default_activity_sid,
            event_callback_url=event_callback_url,
            events_filter=events_filter,
            friendly_name=friendly_name,
            multi_task_enabled=multi_task_enabled,
            timeout_activity_sid=timeout_activity_sid,
            prioritize_queue_order=prioritize_queue_order,
        )

    async def update_async(
        self,
        default_activity_sid: Union[str, object] = values.unset,
        event_callback_url: Union[str, object] = values.unset,
        events_filter: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        multi_task_enabled: Union[bool, object] = values.unset,
        timeout_activity_sid: Union[str, object] = values.unset,
        prioritize_queue_order: Union[
            "WorkspaceInstance.QueueOrder", object
        ] = values.unset,
    ) -> "WorkspaceInstance":
        """
        Asynchronous coroutine to update the WorkspaceInstance

        :param default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        """
        return await self._proxy.update_async(
            default_activity_sid=default_activity_sid,
            event_callback_url=event_callback_url,
            events_filter=events_filter,
            friendly_name=friendly_name,
            multi_task_enabled=multi_task_enabled,
            timeout_activity_sid=timeout_activity_sid,
            prioritize_queue_order=prioritize_queue_order,
        )

    @property
    def activities(self) -> ActivityList:
        """
        Access the activities
        """
        return self._proxy.activities

    @property
    def events(self) -> EventList:
        """
        Access the events
        """
        return self._proxy.events

    @property
    def tasks(self) -> TaskList:
        """
        Access the tasks
        """
        return self._proxy.tasks

    @property
    def task_channels(self) -> TaskChannelList:
        """
        Access the task_channels
        """
        return self._proxy.task_channels

    @property
    def task_queues(self) -> TaskQueueList:
        """
        Access the task_queues
        """
        return self._proxy.task_queues

    @property
    def workers(self) -> WorkerList:
        """
        Access the workers
        """
        return self._proxy.workers

    @property
    def workflows(self) -> WorkflowList:
        """
        Access the workflows
        """
        return self._proxy.workflows

    @property
    def cumulative_statistics(self) -> WorkspaceCumulativeStatisticsList:
        """
        Access the cumulative_statistics
        """
        return self._proxy.cumulative_statistics

    @property
    def real_time_statistics(self) -> WorkspaceRealTimeStatisticsList:
        """
        Access the real_time_statistics
        """
        return self._proxy.real_time_statistics

    @property
    def statistics(self) -> WorkspaceStatisticsList:
        """
        Access the statistics
        """
        return self._proxy.statistics

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Taskrouter.V1.WorkspaceInstance {}>".format(context)


class WorkspaceContext(InstanceContext):

    def __init__(self, version: Version, sid: str):
        """
        Initialize the WorkspaceContext

        :param version: Version that contains the resource
        :param sid: The SID of the Workspace resource to update.
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/Workspaces/{sid}".format(**self._solution)

        self._activities: Optional[ActivityList] = None
        self._events: Optional[EventList] = None
        self._tasks: Optional[TaskList] = None
        self._task_channels: Optional[TaskChannelList] = None
        self._task_queues: Optional[TaskQueueList] = None
        self._workers: Optional[WorkerList] = None
        self._workflows: Optional[WorkflowList] = None
        self._cumulative_statistics: Optional[WorkspaceCumulativeStatisticsList] = None
        self._real_time_statistics: Optional[WorkspaceRealTimeStatisticsList] = None
        self._statistics: Optional[WorkspaceStatisticsList] = None

    def delete(self) -> bool:
        """
        Deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        """

        headers = values.of({})

        return self._version.delete(method="DELETE", uri=self._uri, headers=headers)

    async def delete_async(self) -> bool:
        """
        Asynchronous coroutine that deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        """

        headers = values.of({})

        return await self._version.delete_async(
            method="DELETE", uri=self._uri, headers=headers
        )

    def fetch(self) -> WorkspaceInstance:
        """
        Fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = self._version.fetch(method="GET", uri=self._uri, headers=headers)

        return WorkspaceInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self) -> WorkspaceInstance:
        """
        Asynchronous coroutine to fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        """

        headers = values.of({})

        headers["Accept"] = "application/json"

        payload = await self._version.fetch_async(
            method="GET", uri=self._uri, headers=headers
        )

        return WorkspaceInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def update(
        self,
        default_activity_sid: Union[str, object] = values.unset,
        event_callback_url: Union[str, object] = values.unset,
        events_filter: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        multi_task_enabled: Union[bool, object] = values.unset,
        timeout_activity_sid: Union[str, object] = values.unset,
        prioritize_queue_order: Union[
            "WorkspaceInstance.QueueOrder", object
        ] = values.unset,
    ) -> WorkspaceInstance:
        """
        Update the WorkspaceInstance

        :param default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        """

        data = values.of(
            {
                "DefaultActivitySid": default_activity_sid,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "FriendlyName": friendly_name,
                "MultiTaskEnabled": serialize.boolean_to_string(multi_task_enabled),
                "TimeoutActivitySid": timeout_activity_sid,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )
        headers = values.of({})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = self._version.update(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return WorkspaceInstance(self._version, payload, sid=self._solution["sid"])

    async def update_async(
        self,
        default_activity_sid: Union[str, object] = values.unset,
        event_callback_url: Union[str, object] = values.unset,
        events_filter: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        multi_task_enabled: Union[bool, object] = values.unset,
        timeout_activity_sid: Union[str, object] = values.unset,
        prioritize_queue_order: Union[
            "WorkspaceInstance.QueueOrder", object
        ] = values.unset,
    ) -> WorkspaceInstance:
        """
        Asynchronous coroutine to update the WorkspaceInstance

        :param default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        """

        data = values.of(
            {
                "DefaultActivitySid": default_activity_sid,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "FriendlyName": friendly_name,
                "MultiTaskEnabled": serialize.boolean_to_string(multi_task_enabled),
                "TimeoutActivitySid": timeout_activity_sid,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )
        headers = values.of({})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = await self._version.update_async(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return WorkspaceInstance(self._version, payload, sid=self._solution["sid"])

    @property
    def activities(self) -> ActivityList:
        """
        Access the activities
        """
        if self._activities is None:
            self._activities = ActivityList(
                self._version,
                self._solution["sid"],
            )
        return self._activities

    @property
    def events(self) -> EventList:
        """
        Access the events
        """
        if self._events is None:
            self._events = EventList(
                self._version,
                self._solution["sid"],
            )
        return self._events

    @property
    def tasks(self) -> TaskList:
        """
        Access the tasks
        """
        if self._tasks is None:
            self._tasks = TaskList(
                self._version,
                self._solution["sid"],
            )
        return self._tasks

    @property
    def task_channels(self) -> TaskChannelList:
        """
        Access the task_channels
        """
        if self._task_channels is None:
            self._task_channels = TaskChannelList(
                self._version,
                self._solution["sid"],
            )
        return self._task_channels

    @property
    def task_queues(self) -> TaskQueueList:
        """
        Access the task_queues
        """
        if self._task_queues is None:
            self._task_queues = TaskQueueList(
                self._version,
                self._solution["sid"],
            )
        return self._task_queues

    @property
    def workers(self) -> WorkerList:
        """
        Access the workers
        """
        if self._workers is None:
            self._workers = WorkerList(
                self._version,
                self._solution["sid"],
            )
        return self._workers

    @property
    def workflows(self) -> WorkflowList:
        """
        Access the workflows
        """
        if self._workflows is None:
            self._workflows = WorkflowList(
                self._version,
                self._solution["sid"],
            )
        return self._workflows

    @property
    def cumulative_statistics(self) -> WorkspaceCumulativeStatisticsList:
        """
        Access the cumulative_statistics
        """
        if self._cumulative_statistics is None:
            self._cumulative_statistics = WorkspaceCumulativeStatisticsList(
                self._version,
                self._solution["sid"],
            )
        return self._cumulative_statistics

    @property
    def real_time_statistics(self) -> WorkspaceRealTimeStatisticsList:
        """
        Access the real_time_statistics
        """
        if self._real_time_statistics is None:
            self._real_time_statistics = WorkspaceRealTimeStatisticsList(
                self._version,
                self._solution["sid"],
            )
        return self._real_time_statistics

    @property
    def statistics(self) -> WorkspaceStatisticsList:
        """
        Access the statistics
        """
        if self._statistics is None:
            self._statistics = WorkspaceStatisticsList(
                self._version,
                self._solution["sid"],
            )
        return self._statistics

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Taskrouter.V1.WorkspaceContext {}>".format(context)


class WorkspacePage(Page):

    def get_instance(self, payload: Dict[str, Any]) -> WorkspaceInstance:
        """
        Build an instance of WorkspaceInstance

        :param payload: Payload response from the API
        """
        return WorkspaceInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Taskrouter.V1.WorkspacePage>"


class WorkspaceList(ListResource):

    def __init__(self, version: Version):
        """
        Initialize the WorkspaceList

        :param version: Version that contains the resource

        """
        super().__init__(version)

        self._uri = "/Workspaces"

    def create(
        self,
        friendly_name: str,
        event_callback_url: Union[str, object] = values.unset,
        events_filter: Union[str, object] = values.unset,
        multi_task_enabled: Union[bool, object] = values.unset,
        template: Union[str, object] = values.unset,
        prioritize_queue_order: Union[
            "WorkspaceInstance.QueueOrder", object
        ] = values.unset,
    ) -> WorkspaceInstance:
        """
        Create the WorkspaceInstance

        :param friendly_name: A descriptive string that you create to describe the Workspace resource. It can be up to 64 characters long. For example: `Customer Support` or `2014 Election Campaign`.
        :param event_callback_url: The URL we should call when an event occurs. If provided, the Workspace will publish events to this URL, for example, to collect data for reporting. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param events_filter: The list of Workspace events for which to call event_callback_url. For example, if `EventsFilter=task.created, task.canceled, worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be created as multi-tasking. The default is `true`. Multi-tasking allows Workers to handle multiple Tasks simultaneously. When enabled (`true`), each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param template: An available template name. Can be: `NONE` or `FIFO` and the default is `NONE`. Pre-configures the Workspace with the Workflow and Activities specified in the template. `NONE` will create a Workspace with only a set of default activities. `FIFO` will configure TaskRouter with a set of default activities and a single TaskQueue for first-in, first-out distribution, which can be useful when you are getting started with TaskRouter.
        :param prioritize_queue_order:

        :returns: The created WorkspaceInstance
        """

        data = values.of(
            {
                "FriendlyName": friendly_name,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "MultiTaskEnabled": serialize.boolean_to_string(multi_task_enabled),
                "Template": template,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )
        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = self._version.create(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return WorkspaceInstance(self._version, payload)

    async def create_async(
        self,
        friendly_name: str,
        event_callback_url: Union[str, object] = values.unset,
        events_filter: Union[str, object] = values.unset,
        multi_task_enabled: Union[bool, object] = values.unset,
        template: Union[str, object] = values.unset,
        prioritize_queue_order: Union[
            "WorkspaceInstance.QueueOrder", object
        ] = values.unset,
    ) -> WorkspaceInstance:
        """
        Asynchronously create the WorkspaceInstance

        :param friendly_name: A descriptive string that you create to describe the Workspace resource. It can be up to 64 characters long. For example: `Customer Support` or `2014 Election Campaign`.
        :param event_callback_url: The URL we should call when an event occurs. If provided, the Workspace will publish events to this URL, for example, to collect data for reporting. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param events_filter: The list of Workspace events for which to call event_callback_url. For example, if `EventsFilter=task.created, task.canceled, worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be created as multi-tasking. The default is `true`. Multi-tasking allows Workers to handle multiple Tasks simultaneously. When enabled (`true`), each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param template: An available template name. Can be: `NONE` or `FIFO` and the default is `NONE`. Pre-configures the Workspace with the Workflow and Activities specified in the template. `NONE` will create a Workspace with only a set of default activities. `FIFO` will configure TaskRouter with a set of default activities and a single TaskQueue for first-in, first-out distribution, which can be useful when you are getting started with TaskRouter.
        :param prioritize_queue_order:

        :returns: The created WorkspaceInstance
        """

        data = values.of(
            {
                "FriendlyName": friendly_name,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "MultiTaskEnabled": serialize.boolean_to_string(multi_task_enabled),
                "Template": template,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )
        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Content-Type"] = "application/x-www-form-urlencoded"

        headers["Accept"] = "application/json"

        payload = await self._version.create_async(
            method="POST", uri=self._uri, data=data, headers=headers
        )

        return WorkspaceInstance(self._version, payload)

    def stream(
        self,
        friendly_name: Union[str, object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> Iterator[WorkspaceInstance]:
        """
        Streams WorkspaceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param limit: Upper limit for the number of records to return. stream()
                      guarantees to never return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, stream() will attempt to read the
                          limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(friendly_name=friendly_name, page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self,
        friendly_name: Union[str, object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> AsyncIterator[WorkspaceInstance]:
        """
        Asynchronously streams WorkspaceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param limit: Upper limit for the number of records to return. stream()
                      guarantees to never return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, stream() will attempt to read the
                          limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            friendly_name=friendly_name, page_size=limits["page_size"]
        )

        return self._version.stream_async(page, limits["limit"])

    def list(
        self,
        friendly_name: Union[str, object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[WorkspaceInstance]:
        """
        Lists WorkspaceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param limit: Upper limit for the number of records to return. list() guarantees
                      never to return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, list() will attempt to read the limit
                          with the most efficient page size, i.e. min(limit, 1000)

        :returns: list that will contain up to limit results
        """
        return list(
            self.stream(
                friendly_name=friendly_name,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self,
        friendly_name: Union[str, object] = values.unset,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[WorkspaceInstance]:
        """
        Asynchronously lists WorkspaceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param limit: Upper limit for the number of records to return. list() guarantees
                      never to return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, list() will attempt to read the limit
                          with the most efficient page size, i.e. min(limit, 1000)

        :returns: list that will contain up to limit results
        """
        return [
            record
            async for record in await self.stream_async(
                friendly_name=friendly_name,
                limit=limit,
                page_size=page_size,
            )
        ]

    def page(
        self,
        friendly_name: Union[str, object] = values.unset,
        page_token: Union[str, object] = values.unset,
        page_number: Union[int, object] = values.unset,
        page_size: Union[int, object] = values.unset,
    ) -> WorkspacePage:
        """
        Retrieve a single page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param page_token: PageToken provided by the API
        :param page_number: Page Number, this value is simply for client state
        :param page_size: Number of records to return, defaults to 50

        :returns: Page of WorkspaceInstance
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Accept"] = "application/json"

        response = self._version.page(
            method="GET", uri=self._uri, params=data, headers=headers
        )
        return WorkspacePage(self._version, response)

    async def page_async(
        self,
        friendly_name: Union[str, object] = values.unset,
        page_token: Union[str, object] = values.unset,
        page_number: Union[int, object] = values.unset,
        page_size: Union[int, object] = values.unset,
    ) -> WorkspacePage:
        """
        Asynchronously retrieve a single page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param page_token: PageToken provided by the API
        :param page_number: Page Number, this value is simply for client state
        :param page_size: Number of records to return, defaults to 50

        :returns: Page of WorkspaceInstance
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        headers = values.of({"Content-Type": "application/x-www-form-urlencoded"})

        headers["Accept"] = "application/json"

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data, headers=headers
        )
        return WorkspacePage(self._version, response)

    def get_page(self, target_url: str) -> WorkspacePage:
        """
        Retrieve a specific page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param target_url: API-generated URL for the requested results page

        :returns: Page of WorkspaceInstance
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return WorkspacePage(self._version, response)

    async def get_page_async(self, target_url: str) -> WorkspacePage:
        """
        Asynchronously retrieve a specific page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param target_url: API-generated URL for the requested results page

        :returns: Page of WorkspaceInstance
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return WorkspacePage(self._version, response)

    def get(self, sid: str) -> WorkspaceContext:
        """
        Constructs a WorkspaceContext

        :param sid: The SID of the Workspace resource to update.
        """
        return WorkspaceContext(self._version, sid=sid)

    def __call__(self, sid: str) -> WorkspaceContext:
        """
        Constructs a WorkspaceContext

        :param sid: The SID of the Workspace resource to update.
        """
        return WorkspaceContext(self._version, sid=sid)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Taskrouter.V1.WorkspaceList>"
